@@ -8,6 +8,7 @@ import (
 
 	"github.com/lxc/lxd/shared/logger"
 	"github.com/lxc/lxd/shared/simplestreams"
+	"gopkg.in/macaroon-bakery.v2/httpbakery"
 )
 
 // ConnectionArgs represents a set of common connection properties
@@ -27,11 +28,23 @@ type ConnectionArgs struct {
 	// User agent string
 	UserAgent string
 
+	// Authentication type
+	AuthType string
+
+	// Authentication interactor
+	AuthInteractor httpbakery.Interactor
+
 	// Custom proxy
 	Proxy func(*http.Request) (*url.URL, error)
 
 	// Custom HTTP Client (used as base for the connection)
 	HTTPClient *http.Client
+
+	// Controls whether a client verifies the server's certificate chain and host name.
+	InsecureSkipVerify bool
+
+	// Cookie jar
+	CookieJar http.CookieJar
 }
 
 // ConnectLXD lets you connect to a remote LXD daemon over HTTPs.
@@ -123,7 +136,7 @@ func ConnectSimpleStreams(url string, args *ConnectionArgs) (ImageServer, error)
 	}
 
 	// Setup the HTTP client
-	httpClient, err := tlsHTTPClient(args.HTTPClient, args.TLSClientCert, args.TLSClientKey, args.TLSCA, args.TLSServerCert, args.Proxy)
+	httpClient, err := tlsHTTPClient(args.HTTPClient, args.TLSClientCert, args.TLSClientKey, args.TLSCA, args.TLSServerCert, args.InsecureSkipVerify, args.Proxy)
 	if err != nil {
 		return nil, err
 	}
@@ -145,18 +158,30 @@ func httpsLXD(url string, args *ConnectionArgs) (ContainerServer, error) {
 
 	// Initialize the client struct
 	server := ProtocolLXD{
-		httpCertificate: args.TLSServerCert,
-		httpHost:        url,
-		httpProtocol:    "https",
-		httpUserAgent:   args.UserAgent,
+		httpCertificate:  args.TLSServerCert,
+		httpHost:         url,
+		httpProtocol:     "https",
+		httpUserAgent:    args.UserAgent,
+		bakeryInteractor: args.AuthInteractor,
+	}
+	if args.AuthType == "macaroons" {
+		server.RequireAuthenticated(true)
 	}
 
 	// Setup the HTTP client
-	httpClient, err := tlsHTTPClient(args.HTTPClient, args.TLSClientCert, args.TLSClientKey, args.TLSCA, args.TLSServerCert, args.Proxy)
+	httpClient, err := tlsHTTPClient(args.HTTPClient, args.TLSClientCert, args.TLSClientKey, args.TLSCA, args.TLSServerCert, args.InsecureSkipVerify, args.Proxy)
 	if err != nil {
 		return nil, err
 	}
+
+	if args.CookieJar != nil {
+		httpClient.Jar = args.CookieJar
+	}
+
 	server.http = httpClient
+	if args.AuthType == "macaroons" {
+		server.setupBakeryClient()
+	}
 
 	// Test the connection and seed the server information
 	_, _, err = server.GetServer()
