@@ -55,6 +55,12 @@ func (c *execCmd) usage() string {
 
 Execute commands in containers.
 
+The command is executed directly using exec, so there is no shell and shell patterns (variables, file redirects, ...)
+won't be understood. If you need a shell environment you need to execute the shell executeable, passing the shell commands
+as arguments, for example:
+
+    lxc exec <container> -- sh -c "cd /tmp && pwd"
+
 Mode defaults to non-interactive, interactive mode is selected if both stdin AND stdout are terminals (stderr is ignored).`)
 }
 
