@@ -5,6 +5,8 @@ package main
 import (
 	"io"
 	"os"
+	"os/signal"
+	"syscall"
 
 	"github.com/gorilla/websocket"
 	"github.com/mattn/go-colorable"
@@ -32,11 +34,24 @@ func (c *execCmd) getTERM() (string, bool) {
 }
 
 func (c *execCmd) controlSocketHandler(control *websocket.Conn) {
-	// TODO: figure out what the equivalent of signal.SIGWINCH is on
-	// windows and use that; for now if you resize your terminal it just
-	// won't work quite correctly.
-	err := c.sendTermSize(control)
-	if err != nil {
-		logger.Debugf("error setting term size %s", err)
+	ch := make(chan os.Signal, 10)
+	signal.Notify(ch, os.Interrupt)
+
+	closeMsg := websocket.FormatCloseMessage(websocket.CloseNormalClosure, "")
+	defer control.WriteMessage(websocket.CloseMessage, closeMsg)
+
+	for {
+		sig := <-ch
+		switch sig {
+		case os.Interrupt:
+			logger.Debugf("Received '%s signal', forwarding to executing program.", sig)
+			err := c.forwardSignal(control, syscall.SIGINT)
+			if err != nil {
+				logger.Debugf("Failed to forward signal '%s'.", syscall.SIGINT)
+				return
+			}
+		default:
+			break
+		}
 	}
 }
