@@ -78,6 +78,8 @@ func (c *fileCmd) recursivePullFile(d lxd.ContainerServer, container string, p s
 	}
 
 	target := filepath.Join(targetDir, filepath.Base(p))
+	logger.Infof("Pulling %s from %s (%s)", target, p, resp.Type)
+
 	if resp.Type == "directory" {
 		err := os.Mkdir(target, os.FileMode(resp.Mode))
 		if err != nil {
@@ -163,6 +165,7 @@ func (c *fileCmd) recursivePushFile(d lxd.ContainerServer, container string, sou
 			args.Content = f
 		}
 
+		logger.Infof("Pushing %s to %s (%s)", p, targetPath, args.Type)
 		return d.CreateContainerFile(container, targetPath, args)
 	}
 
@@ -202,6 +205,8 @@ func (c *fileCmd) recursiveMkdir(d lxd.ContainerServer, container string, p stri
 			continue
 		}
 
+		cur = "/" + cur
+
 		modeArg := -1
 		if mode != nil {
 			modeArg = int(mode.Perm())
@@ -213,6 +218,7 @@ func (c *fileCmd) recursiveMkdir(d lxd.ContainerServer, container string, p stri
 			Type: "directory",
 		}
 
+		logger.Infof("Creating %s (%s)", cur, args.Type)
 		err := d.CreateContainerFile(container, cur, args)
 		if err != nil {
 			return err
@@ -250,8 +256,6 @@ func (c *fileCmd) push(conf *config.Config, send_file_perms bool, args []string)
 		targetIsDir = true
 	}
 
-	logger.Debugf("Pushing to: %s  (isdir: %t)", targetPath, targetIsDir)
-
 	d, err := conf.GetContainerServer(remote)
 	if err != nil {
 		return err
@@ -260,7 +264,7 @@ func (c *fileCmd) push(conf *config.Config, send_file_perms bool, args []string)
 	var sourcefilenames []string
 	for _, fname := range args[:len(args)-1] {
 		if !strings.HasPrefix(fname, "--") {
-			sourcefilenames = append(sourcefilenames, fname)
+			sourcefilenames = append(sourcefilenames, shared.HostPath(filepath.Clean(fname)))
 		}
 	}
 
@@ -408,7 +412,9 @@ func (c *fileCmd) push(conf *config.Config, send_file_perms bool, args []string)
 			args.GID = int64(gid)
 			args.Mode = int(mode.Perm())
 		}
+		args.Type = "file"
 
+		logger.Infof("Pushing %s to %s (%s)", f.Name(), fpath, args.Type)
 		err = d.CreateContainerFile(container, fpath, args)
 		if err != nil {
 			return err
@@ -423,7 +429,7 @@ func (c *fileCmd) pull(conf *config.Config, args []string) error {
 		return errArgs
 	}
 
-	target := args[len(args)-1]
+	target := shared.HostPath(filepath.Clean(args[len(args)-1]))
 	targetIsDir := false
 	sb, err := os.Stat(target)
 	if err != nil && !os.IsNotExist(err) {
@@ -492,6 +498,8 @@ func (c *fileCmd) pull(conf *config.Config, args []string) error {
 			targetPath = target
 		}
 
+		logger.Infof("Pulling %s from %s (%s)", targetPath, pathSpec[1], resp.Type)
+
 		var f *os.File
 		if targetPath == "-" {
 			f = os.Stdout
@@ -563,6 +571,9 @@ func (c *fileCmd) edit(conf *config.Config, args []string) error {
 
 	// Create temp file
 	f, err := ioutil.TempFile("", "lxd_file_edit_")
+	if err != nil {
+		return fmt.Errorf("Unable to create a temporary file: %v", err)
+	}
 	fname := f.Name()
 	f.Close()
 	os.Remove(fname)
