@@ -61,11 +61,12 @@ func (f *profileList) Set(value string) error {
 var initRequestedEmptyProfiles bool
 
 type initCmd struct {
-	profArgs    profileList
-	confArgs    configList
-	ephem       bool
-	network     string
-	storagePool string
+	profArgs     profileList
+	confArgs     configList
+	ephem        bool
+	network      string
+	storagePool  string
+	instanceType string
 }
 
 func (c *initCmd) showByDefault() bool {
@@ -74,7 +75,7 @@ func (c *initCmd) showByDefault() bool {
 
 func (c *initCmd) usage() string {
 	return i18n.G(
-		`Usage: lxc init [<remote>:]<image> [<remote>:][<name>] [--ephemeral|-e] [--profile|-p <profile>...] [--config|-c <key=value>...] [--network|-n <network>] [--storage|-s <pool>]
+		`Usage: lxc init [<remote>:]<image> [<remote>:][<name>] [--ephemeral|-e] [--profile|-p <profile>...] [--config|-c <key=value>...] [--network|-n <network>] [--storage|-s <pool>] [--type|-t <instance type>]
 
 Create containers from images.
 
@@ -143,6 +144,7 @@ func (c *initCmd) flags() {
 	gnuflag.StringVar(&c.network, "n", "", i18n.G("Network name"))
 	gnuflag.StringVar(&c.storagePool, "storage", "", i18n.G("Storage pool name"))
 	gnuflag.StringVar(&c.storagePool, "s", "", i18n.G("Storage pool name"))
+	gnuflag.StringVar(&c.instanceType, "t", "", i18n.G("Instance type"))
 }
 
 func (c *initCmd) run(conf *config.Config, args []string) error {
@@ -244,7 +246,8 @@ func (c *initCmd) create(conf *config.Config, args []string) (lxd.ContainerServe
 
 	// Setup container creation request
 	req := api.ContainersPost{
-		Name: name,
+		Name:         name,
+		InstanceType: c.instanceType,
 	}
 	req.Config = configMap
 	req.Devices = devicesMap
@@ -290,7 +293,7 @@ func (c *initCmd) create(conf *config.Config, args []string) (lxd.ContainerServe
 		return nil, "", err
 	}
 
-	err = op.Wait()
+	err = cancelableWait(op, &progress)
 	if err != nil {
 		progress.Done("")
 		return nil, "", err
@@ -310,7 +313,8 @@ func (c *initCmd) create(conf *config.Config, args []string) (lxd.ContainerServe
 
 	if len(containers) == 1 && name == "" {
 		fields := strings.Split(containers[0], "/")
-		fmt.Printf(i18n.G("Container name is: %s")+"\n", fields[len(fields)-1])
+		name = fields[len(fields)-1]
+		fmt.Printf(i18n.G("Container name is: %s")+"\n", name)
 	}
 
 	// Validate the network setup
