@@ -10,6 +10,8 @@ import (
 	"strings"
 	"syscall"
 
+	"gopkg.in/macaroon-bakery.v2/httpbakery/form"
+
 	"github.com/lxc/lxd/lxc/config"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/gnuflag"
@@ -17,6 +19,8 @@ import (
 	"github.com/lxc/lxd/shared/logger"
 	"github.com/lxc/lxd/shared/logging"
 	"github.com/lxc/lxd/shared/version"
+
+	schemaform "gopkg.in/juju/environschema.v1/form"
 )
 
 var configPath string
@@ -90,17 +94,22 @@ func run() error {
 	var err error
 
 	if *forceLocal {
-		conf = &config.DefaultConfig
+		conf = config.NewConfig("", true)
 	} else if shared.PathExists(configPath) {
 		conf, err = config.LoadConfig(configPath)
 		if err != nil {
 			return err
 		}
 	} else {
-		conf = &config.DefaultConfig
-		conf.ConfigDir = filepath.Dir(configPath)
+		conf = config.NewConfig(filepath.Dir(configPath), true)
 	}
 
+	// Add interactor for external authentication
+	conf.SetAuthInteractor(form.Interactor{Filler: schemaform.IOFiller{}})
+
+	// Save cookies on exit
+	defer conf.SaveCookies()
+
 	// Set the user agent
 	conf.UserAgent = version.UserAgent
 
@@ -146,8 +155,7 @@ func run() error {
 	// and this is the first time the client has been run by the user, then check to see
 	// if LXD has been properly configured.  Don't display the message if the var path
 	// does not exist (LXD not installed), as the user may be targeting a remote daemon.
-	if os.Args[0] != "help" && os.Args[0] != "version" && shared.PathExists(shared.VarPath("")) && !shared.PathExists(conf.ConfigDir) {
-
+	if os.Args[0] != "help" && os.Args[0] != "version" && shared.PathExists(shared.VarPath("")) && !shared.PathExists(configPath) {
 		// Create the config dir so that we don't get in here again for this user.
 		err = os.MkdirAll(conf.ConfigDir, 0750)
 		if err != nil {
@@ -205,23 +213,26 @@ type command interface {
 }
 
 var commands = map[string]command{
-	"config":  &configCmd{},
-	"copy":    &copyCmd{},
-	"delete":  &deleteCmd{},
-	"exec":    &execCmd{},
-	"file":    &fileCmd{},
-	"finger":  &fingerCmd{},
-	"query":   &queryCmd{},
-	"help":    &helpCmd{},
-	"image":   &imageCmd{},
-	"info":    &infoCmd{},
-	"init":    &initCmd{},
-	"launch":  &launchCmd{},
-	"list":    &listCmd{},
-	"manpage": &manpageCmd{},
-	"monitor": &monitorCmd{},
-	"move":    &moveCmd{},
-	"network": &networkCmd{},
+	"config":    &configCmd{},
+	"console":   &consoleCmd{},
+	"copy":      &copyCmd{},
+	"delete":    &deleteCmd{},
+	"exec":      &execCmd{},
+	"file":      &fileCmd{},
+	"finger":    &fingerCmd{},
+	"query":     &queryCmd{},
+	"help":      &helpCmd{},
+	"image":     &imageCmd{},
+	"info":      &infoCmd{},
+	"init":      &initCmd{},
+	"launch":    &launchCmd{},
+	"list":      &listCmd{},
+	"manpage":   &manpageCmd{},
+	"monitor":   &monitorCmd{},
+	"rename":    &renameCmd{},
+	"move":      &moveCmd{},
+	"network":   &networkCmd{},
+	"operation": &operationCmd{},
 	"pause": &actionCmd{
 		action:      shared.Freeze,
 		description: i18n.G("Pause containers."),
@@ -261,13 +272,12 @@ var commands = map[string]command{
 // defaultAliases contains LXC's built-in command line aliases.  The built-in
 // aliases are checked only if no user-defined alias was found.
 var defaultAliases = map[string]string{
-	"shell": "exec @ARGS@ -- login -f root",
+	"shell": "exec @ARGS@ -- su -l",
 
-	"cp":     "copy",
-	"ls":     "list",
-	"mv":     "move",
-	"rename": "move",
-	"rm":     "delete",
+	"cp": "copy",
+	"ls": "list",
+	"mv": "move",
+	"rm": "delete",
 
 	"image cp": "image copy",
 	"image ls": "image list",
