@@ -12,6 +12,8 @@ import (
 
 type moveCmd struct {
 	containerOnly bool
+	mode          string
+	stateless     bool
 }
 
 func (c *moveCmd) showByDefault() bool {
@@ -36,6 +38,8 @@ lxc move <container>/<old snapshot name> <container>/<new snapshot name>
 
 func (c *moveCmd) flags() {
 	gnuflag.BoolVar(&c.containerOnly, "container-only", false, i18n.G("Move the container without its snapshots"))
+	gnuflag.StringVar(&c.mode, "mode", "pull", i18n.G("Transfer mode. One of pull (default), push or relay."))
+	gnuflag.BoolVar(&c.stateless, "stateless", false, i18n.G("Copy a stateful container stateless"))
 }
 
 func (c *moveCmd) run(conf *config.Config, args []string) error {
@@ -43,6 +47,12 @@ func (c *moveCmd) run(conf *config.Config, args []string) error {
 		return errArgs
 	}
 
+	// Parse the mode
+	mode := "pull"
+	if c.mode != "" {
+		mode = c.mode
+	}
+
 	sourceRemote, sourceName, err := conf.ParseRemote(args[0])
 	if err != nil {
 		return err
@@ -88,12 +98,16 @@ func (c *moveCmd) run(conf *config.Config, args []string) error {
 
 	cpy := copyCmd{}
 
+	stateful := !c.stateless
+
 	// A move is just a copy followed by a delete; however, we want to
 	// keep the volatile entries around since we are moving the container.
-	err = cpy.copyContainer(conf, args[0], args[1], true, -1, true, c.containerOnly)
+	err = cpy.copyContainer(conf, args[0], args[1], true, -1, stateful, c.containerOnly, mode)
 	if err != nil {
 		return err
 	}
 
-	return commands["delete"].run(conf, args[:1])
+	del := deleteCmd{}
+	del.force = true
+	return del.run(conf, args[:1])
 }
