@@ -5,6 +5,8 @@ import (
 	"testing"
 
 	"github.com/stretchr/testify/suite"
+
+	"github.com/lxc/lxd/shared/api"
 )
 
 type utilsTestSuite struct {
@@ -35,3 +37,23 @@ func (s *utilsTestSuite) Test_StringList_empty_strings() {
 	sort.Sort(StringList(data))
 	s.Equal([][]string{{"foo", "baz"}, {"", "bar"}}, data)
 }
+
+func (s *utilsTestSuite) TestGetExistingAliases() {
+	images := []api.ImageAliasesEntry{
+		{Name: "foo"},
+		{Name: "bar"},
+		{Name: "baz"},
+	}
+	aliases := GetExistingAliases([]string{"bar", "foo", "other"}, images)
+	s.Exactly([]api.ImageAliasesEntry{images[0], images[1]}, aliases)
+}
+
+func (s *utilsTestSuite) TestGetExistingAliasesEmpty() {
+	images := []api.ImageAliasesEntry{
+		{Name: "foo"},
+		{Name: "bar"},
+		{Name: "baz"},
+	}
+	aliases := GetExistingAliases([]string{"other1", "other2"}, images)
+	s.Exactly([]api.ImageAliasesEntry{}, aliases)
+}
