@@ -1,12 +1,10 @@
 package main
 
 import (
-	"encoding/pem"
 	"fmt"
 	"net/http"
 	"os"
 	"reflect"
-	"syscall"
 
 	"gopkg.in/lxc/go-lxc.v2"
 
@@ -21,6 +19,7 @@ import (
 var api10 = []Command{
 	containersCmd,
 	containerCmd,
+	containerConsoleCmd,
 	containerStateCmd,
 	containerFileCmd,
 	containerLogsCmd,
@@ -49,142 +48,50 @@ var api10 = []Command{
 	certificateFingerprintCmd,
 	profilesCmd,
 	profileCmd,
+	serverResourceCmd,
 	storagePoolsCmd,
 	storagePoolCmd,
+	storagePoolResourcesCmd,
 	storagePoolVolumesCmd,
 	storagePoolVolumesTypeCmd,
 	storagePoolVolumeTypeCmd,
+	serverResourceCmd,
 }
 
 func api10Get(d *Daemon, r *http.Request) Response {
+	authMethods := []string{"tls"}
+	if daemonConfig["core.macaroon.endpoint"].Get() != "" {
+		authMethods = append(authMethods, "macaroons")
+	}
 	srv := api.ServerUntrusted{
-		/* List of API extensions in the order they were added.
-		 *
-		 * The following kind of changes require an addition to api_extensions:
-		 *  - New configuration key
-		 *  - New valid values for a configuration key
-		 *  - New REST API endpoint
-		 *  - New argument inside an existing REST API call
-		 *  - New HTTPs authentication mechanisms or protocols
-		 */
-		APIExtensions: []string{
-			"storage_zfs_remove_snapshots",
-			"container_host_shutdown_timeout",
-			"container_syscall_filtering",
-			"auth_pki",
-			"container_last_used_at",
-			"etag",
-			"patch",
-			"usb_devices",
-			"https_allowed_credentials",
-			"image_compression_algorithm",
-			"directory_manipulation",
-			"container_cpu_time",
-			"storage_zfs_use_refquota",
-			"storage_lvm_mount_options",
-			"network",
-			"profile_usedby",
-			"container_push",
-			"container_exec_recording",
-			"certificate_update",
-			"container_exec_signal_handling",
-			"gpu_devices",
-			"container_image_properties",
-			"migration_progress",
-			"id_map",
-			"network_firewall_filtering",
-			"network_routes",
-			"storage",
-			"file_delete",
-			"file_append",
-			"network_dhcp_expiry",
-			"storage_lvm_vg_rename",
-			"storage_lvm_thinpool_rename",
-			"network_vlan",
-			"image_create_aliases",
-			"container_stateless_copy",
-			"container_only_migration",
-			"storage_zfs_clone_copy",
-			"unix_device_rename",
-			"storage_lvm_use_thinpool",
-			"storage_rsync_bwlimit",
-			"network_vxlan_interface",
-			"storage_btrfs_mount_options",
-			"entity_description",
-			"image_force_refresh",
-			"storage_lvm_lv_resizing",
-			"id_map_base",
-			"file_symlinks",
-			"container_push_target",
-			"network_vlan_physical",
-			"storage_images_delete",
-			"container_edit_metadata",
-			"container_snapshot_stateful_migration",
-			"storage_driver_ceph",
-			"storage_ceph_user_name",
-			"resource_limits",
-			"storage_volatile_initial_source",
-			"storage_ceph_force_osd_reuse",
-			"storage_block_filesystem_btrfs",
-		},
-		APIStatus:  "stable",
-		APIVersion: version.APIVersion,
-		Public:     false,
-		Auth:       "untrusted",
+		APIExtensions: version.APIExtensions,
+		APIStatus:     "stable",
+		APIVersion:    version.APIVersion,
+		Public:        false,
+		Auth:          "untrusted",
+		AuthMethods:   authMethods,
 	}
 
 	// If untrusted, return now
-	if !util.IsTrustedClient(r, d.clientCerts) {
+	if d.checkTrustedClient(r) != nil {
 		return SyncResponseETag(true, srv, nil)
 	}
 
 	srv.Auth = "trusted"
 
-	/*
-	 * Based on: https://groups.google.com/forum/#!topic/golang-nuts/Jel8Bb-YwX8
-	 * there is really no better way to do this, which is
-	 * unfortunate. Also, we ditch the more accepted CharsToString
-	 * version in that thread, since it doesn't seem as portable,
-	 * viz. github issue #206.
-	 */
-	uname := syscall.Utsname{}
-	if err := syscall.Uname(&uname); err != nil {
+	uname, err := shared.Uname()
+	if err != nil {
 		return InternalError(err)
 	}
 
-	kernel := ""
-	for _, c := range uname.Sysname {
-		if c == 0 {
-			break
-		}
-		kernel += string(byte(c))
-	}
-
-	kernelVersion := ""
-	for _, c := range uname.Release {
-		if c == 0 {
-			break
-		}
-		kernelVersion += string(byte(c))
-	}
-
-	kernelArchitecture := ""
-	for _, c := range uname.Machine {
-		if c == 0 {
-			break
-		}
-		kernelArchitecture += string(byte(c))
-	}
-
 	addresses, err := util.ListenAddresses(daemonConfig["core.https_address"].Get())
 	if err != nil {
 		return InternalError(err)
 	}
 
-	var certificate string
+	certificate := string(d.endpoints.NetworkPublicKey())
 	var certificateFingerprint string
-	if len(d.tlsConfig.Certificates) != 0 {
-		certificate = string(pem.EncodeToMemory(&pem.Block{Type: "CERTIFICATE", Bytes: d.tlsConfig.Certificates[0].Certificate[0]}))
+	if certificate != "" {
 		certificateFingerprint, err = shared.CertFingerprintStr(certificate)
 		if err != nil {
 			return InternalError(err)
@@ -208,21 +115,15 @@ func api10Get(d *Daemon, r *http.Request) Response {
 		CertificateFingerprint: certificateFingerprint,
 		Driver:                 "lxc",
 		DriverVersion:          lxc.Version(),
-		Kernel:                 kernel,
-		KernelArchitecture:     kernelArchitecture,
-		KernelVersion:          kernelVersion,
+		Kernel:                 uname.Sysname,
+		KernelArchitecture:     uname.Machine,
+		KernelVersion:          uname.Release,
 		Server:                 "lxd",
 		ServerPid:              os.Getpid(),
 		ServerVersion:          version.Version}
 
 	drivers := readStoragePoolDriversCache()
-	for _, driver := range drivers {
-		// Initialize a core storage interface for the given driver.
-		sCore, err := storageCoreInit(driver)
-		if err != nil {
-			continue
-		}
-
+	for driver, version := range drivers {
 		if env.Storage != "" {
 			env.Storage = env.Storage + " | " + driver
 		} else {
@@ -230,11 +131,10 @@ func api10Get(d *Daemon, r *http.Request) Response {
 		}
 
 		// Get the version of the storage drivers in use.
-		sVersion := sCore.GetStorageTypeVersion()
 		if env.StorageVersion != "" {
-			env.StorageVersion = env.StorageVersion + " | " + sVersion
+			env.StorageVersion = env.StorageVersion + " | " + version
 		} else {
-			env.StorageVersion = sVersion
+			env.StorageVersion = version
 		}
 	}
 
@@ -246,7 +146,7 @@ func api10Get(d *Daemon, r *http.Request) Response {
 }
 
 func api10Put(d *Daemon, r *http.Request) Response {
-	oldConfig, err := db.ConfigValuesGet(d.db)
+	oldConfig, err := db.ConfigValuesGet(d.db.DB())
 	if err != nil {
 		return SmartError(err)
 	}
@@ -265,7 +165,7 @@ func api10Put(d *Daemon, r *http.Request) Response {
 }
 
 func api10Patch(d *Daemon, r *http.Request) Response {
-	oldConfig, err := db.ConfigValuesGet(d.db)
+	oldConfig, err := db.ConfigValuesGet(d.db.DB())
 	if err != nil {
 		return SmartError(err)
 	}
