@@ -1,7 +1,6 @@
 package main
 
 import (
-	"database/sql"
 	"fmt"
 	"io"
 	"os"
@@ -81,6 +80,8 @@ func containerValidConfigKey(os *sys.OS, key string, value string) error {
 	return nil
 }
 
+var containerNetworkLimitKeys = []string{"limits.max", "limits.ingress", "limits.egress"}
+
 func containerValidDeviceConfigKey(t, k string) bool {
 	if k == "type" {
 		return true
@@ -134,6 +135,10 @@ func containerValidDeviceConfigKey(t, k string) bool {
 			return true
 		case "security.mac_filtering":
 			return true
+		case "maas.subnet.ipv4":
+			return true
+		case "maas.subnet.ipv6":
+			return true
 		default:
 			return false
 		}
@@ -198,6 +203,21 @@ func containerValidDeviceConfigKey(t, k string) bool {
 		default:
 			return false
 		}
+	case "infiniband":
+		switch k {
+		case "hwaddr":
+			return true
+		case "mtu":
+			return true
+		case "name":
+			return true
+		case "nictype":
+			return true
+		case "parent":
+			return true
+		default:
+			return false
+		}
 	case "none":
 		return false
 	default:
@@ -276,7 +296,7 @@ func containerGetRootDiskDevice(devices types.Devices) (string, types.Device, er
 	return "", types.Device{}, fmt.Errorf("No root device could be found.")
 }
 
-func containerValidDevices(dbObj *sql.DB, devices types.Devices, profile bool, expanded bool) error {
+func containerValidDevices(db *db.Node, devices types.Devices, profile bool, expanded bool) error {
 	// Empty device list
 	if devices == nil {
 		return nil
@@ -289,7 +309,7 @@ func containerValidDevices(dbObj *sql.DB, devices types.Devices, profile bool, e
 			return fmt.Errorf("Missing device type for device '%s'", name)
 		}
 
-		if !shared.StringInSlice(m["type"], []string{"none", "nic", "disk", "unix-char", "unix-block", "usb", "gpu"}) {
+		if !shared.StringInSlice(m["type"], []string{"disk", "gpu", "infiniband", "nic", "none", "unix-block", "unix-char", "usb"}) {
 			return fmt.Errorf("Invalid device type for device '%s'", name)
 		}
 
@@ -304,12 +324,24 @@ func containerValidDevices(dbObj *sql.DB, devices types.Devices, profile bool, e
 				return fmt.Errorf("Missing nic type")
 			}
 
-			if !shared.StringInSlice(m["nictype"], []string{"bridged", "physical", "p2p", "macvlan"}) {
+			if !shared.StringInSlice(m["nictype"], []string{"bridged", "macvlan", "p2p", "physical", "sriov"}) {
 				return fmt.Errorf("Bad nic type: %s", m["nictype"])
 			}
 
-			if shared.StringInSlice(m["nictype"], []string{"bridged", "physical", "macvlan"}) && m["parent"] == "" {
-				return fmt.Errorf("Missing parent for %s type nic.", m["nictype"])
+			if shared.StringInSlice(m["nictype"], []string{"bridged", "macvlan", "physical", "sriov"}) && m["parent"] == "" {
+				return fmt.Errorf("Missing parent for %s type nic", m["nictype"])
+			}
+		} else if m["type"] == "infiniband" {
+			if m["nictype"] == "" {
+				return fmt.Errorf("Missing nic type")
+			}
+
+			if !shared.StringInSlice(m["nictype"], []string{"physical", "sriov"}) {
+				return fmt.Errorf("Bad nic type: %s", m["nictype"])
+			}
+
+			if m["parent"] == "" {
+				return fmt.Errorf("Missing parent for %s type nic", m["nictype"])
 			}
 		} else if m["type"] == "disk" {
 			if !expanded && !shared.StringInSlice(m["path"], diskDevicePaths) {
@@ -343,7 +375,7 @@ func containerValidDevices(dbObj *sql.DB, devices types.Devices, profile bool, e
 					return fmt.Errorf("Storage volumes cannot be specified as absolute paths.")
 				}
 
-				_, err := db.StoragePoolGetID(dbObj, m["pool"])
+				_, err := db.StoragePoolGetID(m["pool"])
 				if err != nil {
 					return fmt.Errorf("The \"%s\" storage pool doesn't exist.", m["pool"])
 				}
@@ -415,7 +447,7 @@ type container interface {
 	/* actionScript here is a script called action.sh in the stateDir, to
 	 * be passed to CRIU as --action-script
 	 */
-	Migrate(cmd uint, stateDir string, function string, stop bool, actionScript bool) error
+	Migrate(args *CriuMigrationArgs) error
 	Snapshots() ([]container, error)
 
 	// Config handling
@@ -436,6 +468,14 @@ type container interface {
 	FilePush(type_ string, srcpath string, dstpath string, uid int64, gid int64, mode int, write string) error
 	FileRemove(path string) error
 
+	// Console - Allocate and run a console tty.
+	//
+	// terminal  - Bidirectional file descriptor.
+	//
+	// This function will not return until the console has been exited by
+	// the user.
+	Console(terminal *os.File) *exec.Cmd
+	ConsoleLog(opts lxc.ConsoleLogOptions) (string, error)
 	/* Command execution:
 		 * 1. passing in false for wait
 		 *    - equivalent to calling cmd.Run()
@@ -486,6 +526,7 @@ type container interface {
 	TemplatesPath() string
 	StatePath() string
 	LogFilePath() string
+	ConsoleBufferLogPath() string
 	LogPath() string
 
 	StoragePool() (string, error)
@@ -498,7 +539,7 @@ type container interface {
 	IdmapSet() (*idmap.IdmapSet, error)
 	LastIdmapSet() (*idmap.IdmapSet, error)
 	TemplateApply(trigger string) error
-	StateObject() *state.State
+	DaemonState() *state.State
 }
 
 // Loader functions
@@ -512,7 +553,7 @@ func containerCreateAsEmpty(d *Daemon, args db.ContainerArgs) (container, error)
 	// Now create the empty storage
 	err = c.Storage().ContainerCreate(c)
 	if err != nil {
-		db.ContainerRemove(d.db, args.Name)
+		d.db.ContainerRemove(args.Name)
 		return nil, err
 	}
 
@@ -536,7 +577,7 @@ func containerCreateEmptySnapshot(s *state.State, args db.ContainerArgs) (contai
 	// Now create the empty snapshot
 	err = c.Storage().ContainerSnapshotCreateEmpty(c)
 	if err != nil {
-		db.ContainerRemove(s.DB, args.Name)
+		s.DB.ContainerRemove(args.Name)
 		return nil, err
 	}
 
@@ -545,7 +586,7 @@ func containerCreateEmptySnapshot(s *state.State, args db.ContainerArgs) (contai
 
 func containerCreateFromImage(s *state.State, args db.ContainerArgs, hash string) (container, error) {
 	// Get the image properties
-	_, img, err := db.ImageGet(s.DB, hash, false, false)
+	_, img, err := s.DB.ImageGet(hash, false, false)
 	if err != nil {
 		return nil, err
 	}
@@ -566,16 +607,16 @@ func containerCreateFromImage(s *state.State, args db.ContainerArgs, hash string
 		return nil, err
 	}
 
-	err = db.ImageLastAccessUpdate(s.DB, hash, time.Now().UTC())
+	err = s.DB.ImageLastAccessUpdate(hash, time.Now().UTC())
 	if err != nil {
-		db.ContainerRemove(s.DB, args.Name)
+		s.DB.ContainerRemove(args.Name)
 		return nil, fmt.Errorf("Error updating image last use date: %s", err)
 	}
 
 	// Now create the storage from an image
 	err = c.Storage().ContainerCreateFromImage(c, hash)
 	if err != nil {
-		db.ContainerRemove(s.DB, args.Name)
+		s.DB.ContainerRemove(args.Name)
 		return nil, err
 	}
 
@@ -600,7 +641,7 @@ func containerCreateAsCopy(s *state.State, args db.ContainerArgs, sourceContaine
 	if !containerOnly {
 		snapshots, err := sourceContainer.Snapshots()
 		if err != nil {
-			db.ContainerRemove(s.DB, args.Name)
+			s.DB.ContainerRemove(args.Name)
 			return nil, err
 		}
 
@@ -632,9 +673,9 @@ func containerCreateAsCopy(s *state.State, args db.ContainerArgs, sourceContaine
 	err = ct.Storage().ContainerCopy(ct, sourceContainer, containerOnly)
 	if err != nil {
 		for _, v := range csList {
-			db.ContainerRemove(s.DB, (*v).Name())
+			s.DB.ContainerRemove((*v).Name())
 		}
-		db.ContainerRemove(s.DB, args.Name)
+		s.DB.ContainerRemove(args.Name)
 		return nil, err
 	}
 
@@ -687,7 +728,17 @@ func containerCreateAsSnapshot(s *state.State, args db.ContainerArgs, sourceCont
 		 * after snapshotting will fail.
 		 */
 
-		err = sourceContainer.Migrate(lxc.MIGRATE_DUMP, stateDir, "snapshot", false, false)
+		criuMigrationArgs := CriuMigrationArgs{
+			cmd:          lxc.MIGRATE_DUMP,
+			stateDir:     stateDir,
+			function:     "snapshot",
+			stop:         false,
+			actionScript: false,
+			dumpDir:      "",
+			preDumpDir:   "",
+		}
+
+		err = sourceContainer.Migrate(&criuMigrationArgs)
 		if err != nil {
 			os.RemoveAll(sourceContainer.StatePath())
 			return nil, err
@@ -703,7 +754,7 @@ func containerCreateAsSnapshot(s *state.State, args db.ContainerArgs, sourceCont
 	// Clone the container
 	err = sourceContainer.Storage().ContainerSnapshotCreate(c, sourceContainer)
 	if err != nil {
-		db.ContainerRemove(s.DB, args.Name)
+		s.DB.ContainerRemove(args.Name)
 		return nil, err
 	}
 
@@ -782,7 +833,7 @@ func containerCreateInternal(s *state.State, args db.ContainerArgs) (container,
 	}
 
 	// Validate profiles
-	profiles, err := db.Profiles(s.DB)
+	profiles, err := s.DB.Profiles()
 	if err != nil {
 		return nil, err
 	}
@@ -794,7 +845,7 @@ func containerCreateInternal(s *state.State, args db.ContainerArgs) (container,
 	}
 
 	// Create the container entry
-	id, err := db.ContainerCreate(s.DB, args)
+	id, err := s.DB.ContainerCreate(args)
 	if err != nil {
 		if err == db.DbErrAlreadyDefined {
 			thing := "Container"
@@ -812,9 +863,9 @@ func containerCreateInternal(s *state.State, args db.ContainerArgs) (container,
 	args.Id = id
 
 	// Read the timestamp from the database
-	dbArgs, err := db.ContainerGet(s.DB, args.Name)
+	dbArgs, err := s.DB.ContainerGet(args.Name)
 	if err != nil {
-		db.ContainerRemove(s.DB, args.Name)
+		s.DB.ContainerRemove(args.Name)
 		return nil, err
 	}
 	args.CreationDate = dbArgs.CreationDate
@@ -823,7 +874,7 @@ func containerCreateInternal(s *state.State, args db.ContainerArgs) (container,
 	// Setup the container struct and finish creation (storage and idmap)
 	c, err := containerLXCCreate(s, args)
 	if err != nil {
-		db.ContainerRemove(s.DB, args.Name)
+		s.DB.ContainerRemove(args.Name)
 		return nil, err
 	}
 
@@ -878,7 +929,7 @@ func containerConfigureInternal(c container) error {
 
 func containerLoadById(s *state.State, id int) (container, error) {
 	// Get the DB record
-	name, err := db.ContainerName(s.DB, id)
+	name, err := s.DB.ContainerName(id)
 	if err != nil {
 		return nil, err
 	}
@@ -888,7 +939,7 @@ func containerLoadById(s *state.State, id int) (container, error) {
 
 func containerLoadByName(s *state.State, name string) (container, error) {
 	// Get the DB record
-	args, err := db.ContainerGet(s.DB, name)
+	args, err := s.DB.ContainerGet(name)
 	if err != nil {
 		return nil, err
 	}
