@@ -11,8 +11,8 @@ import (
 	"strings"
 	"sync"
 
+	log "github.com/lxc/lxd/shared/log15"
 	"golang.org/x/crypto/scrypt"
-	log "gopkg.in/inconshreveable/log15.v2"
 
 	dbapi "github.com/lxc/lxd/lxd/db"
 	"github.com/lxc/lxd/shared"
@@ -128,11 +128,16 @@ func (k *daemonConfigKey) Set(d *Daemon, value string) error {
 	k.currentValue = value
 	daemonConfigLock.Unlock()
 
-	err = dbapi.ConfigValueSet(d.db, name, value)
+	err = dbapi.ConfigValueSet(d.db.DB(), name, value)
 	if err != nil {
 		return err
 	}
 
+	// Run the trigger (if any)
+	if k.trigger != nil {
+		k.trigger(d, k.name(), value)
+	}
+
 	return nil
 }
 
@@ -184,12 +189,17 @@ func daemonConfigInit(db *sql.DB) error {
 		"core.proxy_https":               {valueType: "string", setter: daemonConfigSetProxy},
 		"core.proxy_ignore_hosts":        {valueType: "string", setter: daemonConfigSetProxy},
 		"core.trust_password":            {valueType: "string", hiddenValue: true, setter: daemonConfigSetPassword},
+		"core.macaroon.endpoint":         {valueType: "string", setter: daemonConfigSetMacaroonEndpoint},
 
 		"images.auto_update_cached":    {valueType: "bool", defaultValue: "true"},
 		"images.auto_update_interval":  {valueType: "int", defaultValue: "6", trigger: daemonConfigTriggerAutoUpdateInterval},
 		"images.compression_algorithm": {valueType: "string", validator: daemonConfigValidateCompression, defaultValue: "gzip"},
 		"images.remote_cache_expiry":   {valueType: "int", defaultValue: "10", trigger: daemonConfigTriggerExpiry},
 
+		"maas.api.key": {valueType: "string", setter: daemonConfigSetMAAS},
+		"maas.api.url": {valueType: "string", setter: daemonConfigSetMAAS},
+		"maas.machine": {valueType: "string", setter: daemonConfigSetMAAS},
+
 		// Keys deprecated since the implementation of the storage api.
 		"storage.lvm_fstype":           {valueType: "string", defaultValue: "ext4", validValues: []string{"btrfs", "ext4", "xfs"}, validator: storageDeprecatedKeys},
 		"storage.lvm_mount_options":    {valueType: "string", defaultValue: "discard", validator: storageDeprecatedKeys},
@@ -265,8 +275,16 @@ func daemonConfigSetPassword(d *Daemon, key string, value string) (string, error
 }
 
 func daemonConfigSetAddress(d *Daemon, key string, value string) (string, error) {
-	// Update the current https address
-	err := d.UpdateHTTPsPort(value)
+	err := d.endpoints.NetworkUpdateAddress(value)
+	if err != nil {
+		return "", err
+	}
+
+	return value, nil
+}
+
+func daemonConfigSetMacaroonEndpoint(d *Daemon, key string, value string) (string, error) {
+	err := d.setupExternalAuthentication(value)
 	if err != nil {
 		return "", err
 	}
@@ -301,14 +319,38 @@ func daemonConfigSetProxy(d *Daemon, key string, value string) (string, error) {
 	return value, nil
 }
 
+func daemonConfigSetMAAS(d *Daemon, key string, value string) (string, error) {
+	maasUrl := daemonConfig["maas.api.url"].Get()
+	if key == "maas.api.url" {
+		maasUrl = value
+	}
+
+	maasKey := daemonConfig["maas.api.key"].Get()
+	if key == "maas.api.key" {
+		maasKey = value
+	}
+
+	maasMachine := daemonConfig["maas.machine"].Get()
+	if key == "maas.machine" {
+		maasMachine = value
+	}
+
+	err := d.setupMAASController(maasUrl, maasKey, maasMachine)
+	if err != nil {
+		return "", err
+	}
+
+	return value, nil
+}
+
 func daemonConfigTriggerExpiry(d *Daemon, key string, value string) {
 	// Trigger an image pruning run
-	d.pruneChan <- true
+	d.taskPruneImages.Reset()
 }
 
 func daemonConfigTriggerAutoUpdateInterval(d *Daemon, key string, value string) {
 	// Reset the auto-update interval loop
-	d.resetAutoUpdateChan <- true
+	d.taskAutoUpdate.Reset()
 }
 
 func daemonConfigValidateCompression(d *Daemon, key string, value string) error {
