@@ -9,7 +9,7 @@ import (
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/logger"
 
-	log "gopkg.in/inconshreveable/log15.v2"
+	log "github.com/lxc/lxd/shared/log15"
 )
 
 // ContainerArgs is a value object holding all db-related details about a
@@ -40,13 +40,13 @@ const (
 	CTypeSnapshot ContainerType = 1
 )
 
-func ContainerRemove(db *sql.DB, name string) error {
-	id, err := ContainerId(db, name)
+func (n *Node) ContainerRemove(name string) error {
+	id, err := n.ContainerId(name)
 	if err != nil {
 		return err
 	}
 
-	_, err = Exec(db, "DELETE FROM containers WHERE id=?", id)
+	_, err = exec(n.db, "DELETE FROM containers WHERE id=?", id)
 	if err != nil {
 		return err
 	}
@@ -54,25 +54,25 @@ func ContainerRemove(db *sql.DB, name string) error {
 	return nil
 }
 
-func ContainerName(db *sql.DB, id int) (string, error) {
+func (n *Node) ContainerName(id int) (string, error) {
 	q := "SELECT name FROM containers WHERE id=?"
 	name := ""
 	arg1 := []interface{}{id}
 	arg2 := []interface{}{&name}
-	err := dbQueryRowScan(db, q, arg1, arg2)
+	err := dbQueryRowScan(n.db, q, arg1, arg2)
 	return name, err
 }
 
-func ContainerId(db *sql.DB, name string) (int, error) {
+func (n *Node) ContainerId(name string) (int, error) {
 	q := "SELECT id FROM containers WHERE name=?"
 	id := -1
 	arg1 := []interface{}{name}
 	arg2 := []interface{}{&id}
-	err := dbQueryRowScan(db, q, arg1, arg2)
+	err := dbQueryRowScan(n.db, q, arg1, arg2)
 	return id, err
 }
 
-func ContainerGet(db *sql.DB, name string) (ContainerArgs, error) {
+func (n *Node) ContainerGet(name string) (ContainerArgs, error) {
 	var used *time.Time // Hold the db-returned time
 	description := sql.NullString{}
 
@@ -84,7 +84,7 @@ func ContainerGet(db *sql.DB, name string) (ContainerArgs, error) {
 	q := "SELECT id, description, architecture, type, ephemeral, stateful, creation_date, last_use_date FROM containers WHERE name=?"
 	arg1 := []interface{}{name}
 	arg2 := []interface{}{&args.Id, &description, &args.Architecture, &args.Ctype, &ephemInt, &statefulInt, &args.CreationDate, &used}
-	err := dbQueryRowScan(db, q, arg1, arg2)
+	err := dbQueryRowScan(n.db, q, arg1, arg2)
 	if err != nil {
 		return args, err
 	}
@@ -109,13 +109,13 @@ func ContainerGet(db *sql.DB, name string) (ContainerArgs, error) {
 		args.LastUsedDate = time.Unix(0, 0).UTC()
 	}
 
-	config, err := ContainerConfig(db, args.Id)
+	config, err := n.ContainerConfig(args.Id)
 	if err != nil {
 		return args, err
 	}
 	args.Config = config
 
-	profiles, err := ContainerProfiles(db, args.Id)
+	profiles, err := n.ContainerProfiles(args.Id)
 	if err != nil {
 		return args, err
 	}
@@ -123,7 +123,7 @@ func ContainerGet(db *sql.DB, name string) (ContainerArgs, error) {
 
 	/* get container_devices */
 	args.Devices = types.Devices{}
-	newdevs, err := Devices(db, name, false)
+	newdevs, err := n.Devices(name, false)
 	if err != nil {
 		return args, err
 	}
@@ -135,13 +135,13 @@ func ContainerGet(db *sql.DB, name string) (ContainerArgs, error) {
 	return args, nil
 }
 
-func ContainerCreate(db *sql.DB, args ContainerArgs) (int, error) {
-	id, err := ContainerId(db, args.Name)
+func (n *Node) ContainerCreate(args ContainerArgs) (int, error) {
+	_, err := n.ContainerId(args.Name)
 	if err == nil {
 		return 0, DbErrAlreadyDefined
 	}
 
-	tx, err := Begin(db)
+	tx, err := begin(n.db)
 	if err != nil {
 		return 0, err
 	}
@@ -178,7 +178,7 @@ func ContainerCreate(db *sql.DB, args ContainerArgs) (int, error) {
 		return 0, fmt.Errorf("Error inserting %s into database", args.Name)
 	}
 	// TODO: is this really int64? we should fix it everywhere if so
-	id = int(id64)
+	id := int(id64)
 	if err := ContainerConfigInsert(tx, id, args.Config); err != nil {
 		tx.Rollback()
 		return 0, err
@@ -238,27 +238,27 @@ func ContainerConfigInsert(tx *sql.Tx, id int, config map[string]string) error {
 	return nil
 }
 
-func ContainerConfigGet(db *sql.DB, id int, key string) (string, error) {
+func (n *Node) ContainerConfigGet(id int, key string) (string, error) {
 	q := "SELECT value FROM containers_config WHERE container_id=? AND key=?"
 	value := ""
 	arg1 := []interface{}{id, key}
 	arg2 := []interface{}{&value}
-	err := dbQueryRowScan(db, q, arg1, arg2)
+	err := dbQueryRowScan(n.db, q, arg1, arg2)
 	return value, err
 }
 
-func ContainerConfigRemove(db *sql.DB, id int, name string) error {
-	_, err := Exec(db, "DELETE FROM containers_config WHERE key=? AND container_id=?", name, id)
+func (n *Node) ContainerConfigRemove(id int, name string) error {
+	_, err := exec(n.db, "DELETE FROM containers_config WHERE key=? AND container_id=?", name, id)
 	return err
 }
 
-func ContainerSetStateful(db *sql.DB, id int, stateful bool) error {
+func (n *Node) ContainerSetStateful(id int, stateful bool) error {
 	statefulInt := 0
 	if stateful {
 		statefulInt = 1
 	}
 
-	_, err := Exec(db, "UPDATE containers SET stateful=? WHERE id=?", statefulInt, id)
+	_, err := exec(n.db, "UPDATE containers SET stateful=? WHERE id=?", statefulInt, id)
 	return err
 }
 
@@ -285,7 +285,7 @@ func ContainerProfilesInsert(tx *sql.Tx, id int, profiles []string) error {
 }
 
 // Get a list of profiles for a given container id.
-func ContainerProfiles(db *sql.DB, containerId int) ([]string, error) {
+func (n *Node) ContainerProfiles(containerId int) ([]string, error) {
 	var name string
 	var profiles []string
 
@@ -297,7 +297,7 @@ func ContainerProfiles(db *sql.DB, containerId int) ([]string, error) {
 	inargs := []interface{}{containerId}
 	outfmt := []interface{}{name}
 
-	results, err := QueryScan(db, query, inargs, outfmt)
+	results, err := queryScan(n.db, query, inargs, outfmt)
 	if err != nil {
 		return nil, err
 	}
@@ -312,7 +312,7 @@ func ContainerProfiles(db *sql.DB, containerId int) ([]string, error) {
 }
 
 // ContainerConfig gets the container configuration map from the DB
-func ContainerConfig(db *sql.DB, containerId int) (map[string]string, error) {
+func (n *Node) ContainerConfig(containerId int) (map[string]string, error) {
 	var key, value string
 	q := `SELECT key, value FROM containers_config WHERE container_id=?`
 
@@ -320,7 +320,7 @@ func ContainerConfig(db *sql.DB, containerId int) (map[string]string, error) {
 	outfmt := []interface{}{key, value}
 
 	// Results is already a slice here, not db Rows anymore.
-	results, err := QueryScan(db, q, inargs, outfmt)
+	results, err := queryScan(n.db, q, inargs, outfmt)
 	if err != nil {
 		return nil, err //SmartError will wrap this and make "not found" errors pretty
 	}
@@ -337,12 +337,12 @@ func ContainerConfig(db *sql.DB, containerId int) (map[string]string, error) {
 	return config, nil
 }
 
-func ContainersList(db *sql.DB, cType ContainerType) ([]string, error) {
+func (n *Node) ContainersList(cType ContainerType) ([]string, error) {
 	q := fmt.Sprintf("SELECT name FROM containers WHERE type=? ORDER BY name")
 	inargs := []interface{}{cType}
 	var container string
 	outfmt := []interface{}{container}
-	result, err := QueryScan(db, q, inargs, outfmt)
+	result, err := queryScan(n.db, q, inargs, outfmt)
 	if err != nil {
 		return nil, err
 	}
@@ -355,8 +355,14 @@ func ContainersList(db *sql.DB, cType ContainerType) ([]string, error) {
 	return ret, nil
 }
 
-func ContainerSetState(db *sql.DB, id int, state string) error {
-	tx, err := Begin(db)
+func (n *Node) ContainersResetState() error {
+	// Reset all container states
+	_, err := exec(n.db, "DELETE FROM containers_config WHERE key='volatile.last_state.power'")
+	return err
+}
+
+func (n *Node) ContainerSetState(id int, state string) error {
+	tx, err := begin(n.db)
 	if err != nil {
 		return err
 	}
@@ -392,8 +398,8 @@ func ContainerSetState(db *sql.DB, id int, state string) error {
 	return TxCommit(tx)
 }
 
-func ContainerRename(db *sql.DB, oldName string, newName string) error {
-	tx, err := Begin(db)
+func (n *Node) ContainerRename(oldName string, newName string) error {
+	tx, err := begin(n.db)
 	if err != nil {
 		return err
 	}
@@ -440,13 +446,13 @@ func ContainerUpdate(tx *sql.Tx, id int, description string, architecture int, e
 	return nil
 }
 
-func ContainerLastUsedUpdate(db *sql.DB, id int, date time.Time) error {
+func (n *Node) ContainerLastUsedUpdate(id int, date time.Time) error {
 	stmt := `UPDATE containers SET last_use_date=? WHERE id=?`
-	_, err := Exec(db, stmt, date, id)
+	_, err := exec(n.db, stmt, date, id)
 	return err
 }
 
-func ContainerGetSnapshots(db *sql.DB, name string) ([]string, error) {
+func (n *Node) ContainerGetSnapshots(name string) ([]string, error) {
 	result := []string{}
 
 	regexp := name + shared.SnapshotDelimiter
@@ -454,7 +460,7 @@ func ContainerGetSnapshots(db *sql.DB, name string) ([]string, error) {
 	q := "SELECT name FROM containers WHERE type=? AND SUBSTR(name,1,?)=?"
 	inargs := []interface{}{CTypeSnapshot, length, regexp}
 	outfmt := []interface{}{name}
-	dbResults, err := QueryScan(db, q, inargs, outfmt)
+	dbResults, err := queryScan(n.db, q, inargs, outfmt)
 	if err != nil {
 		return result, err
 	}
@@ -466,8 +472,44 @@ func ContainerGetSnapshots(db *sql.DB, name string) ([]string, error) {
 	return result, nil
 }
 
+/*
+ * Note, the code below doesn't deal with snapshots of snapshots.
+ * To do that, we'll need to weed out based on # slashes in names
+ */
+func (n *Node) ContainerNextSnapshot(name string) int {
+	base := name + shared.SnapshotDelimiter + "snap"
+	length := len(base)
+	q := fmt.Sprintf("SELECT name FROM containers WHERE type=? AND SUBSTR(name,1,?)=?")
+	var numstr string
+	inargs := []interface{}{CTypeSnapshot, length, base}
+	outfmt := []interface{}{numstr}
+	results, err := queryScan(n.db, q, inargs, outfmt)
+	if err != nil {
+		return 0
+	}
+	max := 0
+
+	for _, r := range results {
+		numstr = r[0].(string)
+		if len(numstr) <= length {
+			continue
+		}
+		substr := numstr[length:]
+		var num int
+		count, err := fmt.Sscanf(substr, "%d", &num)
+		if err != nil || count != 1 {
+			continue
+		}
+		if num >= max {
+			max = num + 1
+		}
+	}
+
+	return max
+}
+
 // Get the storage pool of a given container.
-func ContainerPool(db *sql.DB, containerName string) (string, error) {
+func (n *Node) ContainerPool(containerName string) (string, error) {
 	// Get container storage volume. Since container names are globally
 	// unique, and their storage volumes carry the same name, their storage
 	// volumes are unique too.
@@ -478,7 +520,7 @@ WHERE storage_volumes.name=? AND storage_volumes.type=?`
 	inargs := []interface{}{containerName, StoragePoolVolumeTypeContainer}
 	outargs := []interface{}{&poolName}
 
-	err := dbQueryRowScan(db, query, inargs, outargs)
+	err := dbQueryRowScan(n.db, query, inargs, outargs)
 	if err != nil {
 		if err == sql.ErrNoRows {
 			return "", NoSuchObjectError
