@@ -25,6 +25,8 @@ func dbDeviceTypeToString(t int) (string, error) {
 		return "usb", nil
 	case 6:
 		return "gpu", nil
+	case 7:
+		return "infiniband", nil
 	default:
 		return "", fmt.Errorf("Invalid device type %d", t)
 	}
@@ -46,6 +48,8 @@ func dbDeviceTypeToInt(t string) (int, error) {
 		return 5, nil
 	case "gpu":
 		return 6, nil
+	case "infiniband":
+		return 7, nil
 	default:
 		return -1, fmt.Errorf("Invalid device type %s", t)
 	}
@@ -115,7 +119,7 @@ func dbDeviceConfig(db *sql.DB, id int, isprofile bool) (types.Device, error) {
 		query = `SELECT key, value FROM containers_devices_config WHERE container_device_id=?`
 	}
 
-	results, err := QueryScan(db, query, inargs, outfmt)
+	results, err := queryScan(db, query, inargs, outfmt)
 
 	if err != nil {
 		return newdev, err
@@ -130,7 +134,7 @@ func dbDeviceConfig(db *sql.DB, id int, isprofile bool) (types.Device, error) {
 	return newdev, nil
 }
 
-func Devices(db *sql.DB, qName string, isprofile bool) (types.Devices, error) {
+func (n *Node) Devices(qName string, isprofile bool) (types.Devices, error) {
 	var q string
 	if isprofile {
 		q = `SELECT profiles_devices.id, profiles_devices.name, profiles_devices.type
@@ -147,7 +151,7 @@ func Devices(db *sql.DB, qName string, isprofile bool) (types.Devices, error) {
 	var name, stype string
 	inargs := []interface{}{qName}
 	outfmt := []interface{}{id, name, dtype}
-	results, err := QueryScan(db, q, inargs, outfmt)
+	results, err := queryScan(n.db, q, inargs, outfmt)
 	if err != nil {
 		return nil, err
 	}
@@ -160,7 +164,7 @@ func Devices(db *sql.DB, qName string, isprofile bool) (types.Devices, error) {
 		if err != nil {
 			return nil, err
 		}
-		newdev, err := dbDeviceConfig(db, id, isprofile)
+		newdev, err := dbDeviceConfig(n.db, id, isprofile)
 		if err != nil {
 			return nil, err
 		}
