@@ -11,12 +11,12 @@ import (
 )
 
 // Profiles returns a string list of profiles.
-func Profiles(db *sql.DB) ([]string, error) {
+func (n *Node) Profiles() ([]string, error) {
 	q := fmt.Sprintf("SELECT name FROM profiles")
 	inargs := []interface{}{}
 	var name string
 	outfmt := []interface{}{name}
-	result, err := QueryScan(db, q, inargs, outfmt)
+	result, err := queryScan(n.db, q, inargs, outfmt)
 	if err != nil {
 		return []string{}, err
 	}
@@ -29,24 +29,24 @@ func Profiles(db *sql.DB) ([]string, error) {
 	return response, nil
 }
 
-func ProfileGet(db *sql.DB, name string) (int64, *api.Profile, error) {
+func (n *Node) ProfileGet(name string) (int64, *api.Profile, error) {
 	id := int64(-1)
 	description := sql.NullString{}
 
 	q := "SELECT id, description FROM profiles WHERE name=?"
 	arg1 := []interface{}{name}
 	arg2 := []interface{}{&id, &description}
-	err := dbQueryRowScan(db, q, arg1, arg2)
+	err := dbQueryRowScan(n.db, q, arg1, arg2)
 	if err != nil {
 		return -1, nil, err
 	}
 
-	config, err := ProfileConfig(db, name)
+	config, err := n.ProfileConfig(name)
 	if err != nil {
 		return -1, nil, err
 	}
 
-	devices, err := Devices(db, name, true)
+	devices, err := n.Devices(name, true)
 	if err != nil {
 		return -1, nil, err
 	}
@@ -62,10 +62,10 @@ func ProfileGet(db *sql.DB, name string) (int64, *api.Profile, error) {
 	return id, &profile, nil
 }
 
-func ProfileCreate(db *sql.DB, profile string, description string, config map[string]string,
+func (n *Node) ProfileCreate(profile string, description string, config map[string]string,
 	devices types.Devices) (int64, error) {
 
-	tx, err := Begin(db)
+	tx, err := begin(n.db)
 	if err != nil {
 		return -1, err
 	}
@@ -100,15 +100,15 @@ func ProfileCreate(db *sql.DB, profile string, description string, config map[st
 	return id, nil
 }
 
-func ProfileCreateDefault(db *sql.DB) error {
-	id, _, _ := ProfileGet(db, "default")
+func (n *Node) ProfileCreateDefault() error {
+	id, _, _ := n.ProfileGet("default")
 
 	if id != -1 {
 		// default profile already exists
 		return nil
 	}
 
-	_, err := ProfileCreate(db, "default", "Default LXD profile", map[string]string{}, types.Devices{})
+	_, err := n.ProfileCreate("default", "Default LXD profile", map[string]string{}, types.Devices{})
 	if err != nil {
 		return err
 	}
@@ -117,7 +117,7 @@ func ProfileCreateDefault(db *sql.DB) error {
 }
 
 // Get the profile configuration map from the DB
-func ProfileConfig(db *sql.DB, name string) (map[string]string, error) {
+func (n *Node) ProfileConfig(name string) (map[string]string, error) {
 	var key, value string
 	query := `
         SELECT
@@ -127,7 +127,7 @@ func ProfileConfig(db *sql.DB, name string) (map[string]string, error) {
 		WHERE name=?`
 	inargs := []interface{}{name}
 	outfmt := []interface{}{key, value}
-	results, err := QueryScan(db, query, inargs, outfmt)
+	results, err := queryScan(n.db, query, inargs, outfmt)
 	if err != nil {
 		return nil, fmt.Errorf("Failed to get profile '%s'", name)
 	}
@@ -139,7 +139,7 @@ func ProfileConfig(db *sql.DB, name string) (map[string]string, error) {
 		 */
 		query := "SELECT id FROM profiles WHERE name=?"
 		var id int
-		results, err := QueryScan(db, query, []interface{}{name}, []interface{}{id})
+		results, err := queryScan(n.db, query, []interface{}{name}, []interface{}{id})
 		if err != nil {
 			return nil, err
 		}
@@ -161,13 +161,13 @@ func ProfileConfig(db *sql.DB, name string) (map[string]string, error) {
 	return config, nil
 }
 
-func ProfileDelete(db *sql.DB, name string) error {
-	id, _, err := ProfileGet(db, name)
+func (n *Node) ProfileDelete(name string) error {
+	id, _, err := n.ProfileGet(name)
 	if err != nil {
 		return err
 	}
 
-	_, err = Exec(db, "DELETE FROM profiles WHERE id=?", id)
+	_, err = exec(n.db, "DELETE FROM profiles WHERE id=?", id)
 	if err != nil {
 		return err
 	}
@@ -175,8 +175,8 @@ func ProfileDelete(db *sql.DB, name string) error {
 	return nil
 }
 
-func ProfileUpdate(db *sql.DB, name string, newName string) error {
-	tx, err := Begin(db)
+func (n *Node) ProfileUpdate(name string, newName string) error {
+	tx, err := begin(n.db)
 	if err != nil {
 		return err
 	}
@@ -222,6 +222,9 @@ func ProfileConfigAdd(tx *sql.Tx, id int64, config map[string]string) error {
 	str := fmt.Sprintf("INSERT INTO profiles_config (profile_id, key, value) VALUES(?, ?, ?)")
 	stmt, err := tx.Prepare(str)
 	defer stmt.Close()
+	if err != nil {
+		return err
+	}
 
 	for k, v := range config {
 		_, err = stmt.Exec(id, k, v)
@@ -233,7 +236,7 @@ func ProfileConfigAdd(tx *sql.Tx, id int64, config map[string]string) error {
 	return nil
 }
 
-func ProfileContainersGet(db *sql.DB, profile string) ([]string, error) {
+func (n *Node) ProfileContainersGet(profile string) ([]string, error) {
 	q := `SELECT containers.name FROM containers JOIN containers_profiles
 		ON containers.id == containers_profiles.container_id
 		JOIN profiles ON containers_profiles.profile_id == profiles.id
@@ -244,7 +247,7 @@ func ProfileContainersGet(db *sql.DB, profile string) ([]string, error) {
 	var name string
 	outfmt := []interface{}{name}
 
-	output, err := QueryScan(db, q, inargs, outfmt)
+	output, err := queryScan(n.db, q, inargs, outfmt)
 	if err != nil {
 		return results, err
 	}
@@ -255,3 +258,17 @@ func ProfileContainersGet(db *sql.DB, profile string) ([]string, error) {
 
 	return results, nil
 }
+
+func (n *Node) ProfileCleanupLeftover() error {
+	stmt := `
+DELETE FROM profiles_config WHERE profile_id NOT IN (SELECT id FROM profiles);
+DELETE FROM profiles_devices WHERE profile_id NOT IN (SELECT id FROM profiles);
+DELETE FROM profiles_devices_config WHERE profile_device_id NOT IN (SELECT id FROM profiles_devices);
+`
+	_, err := n.db.Exec(stmt)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
