@@ -7,6 +7,7 @@ import (
 
 	_ "github.com/mattn/go-sqlite3"
 	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
 
 	"github.com/lxc/lxd/lxd/db/query"
 	"github.com/lxc/lxd/lxd/db/schema"
@@ -20,7 +21,9 @@ func TestNewFromMap(t *testing.T) {
 		1: updateCreateTable,
 		2: updateInsertValue,
 	})
-	assert.NoError(t, schema.Ensure(db))
+	initial, err := schema.Ensure(db)
+	assert.NoError(t, err)
+	assert.Equal(t, 0, initial)
 }
 
 // Panic if there are missing versions in the map.
@@ -38,27 +41,42 @@ func TestNewFromMap_MissingVersions(t *testing.T) {
 func TestSchemaEnsure_VersionMoreRecentThanExpected(t *testing.T) {
 	schema, db := newSchemaAndDB(t)
 	schema.Add(updateNoop)
-	assert.NoError(t, schema.Ensure(db))
+	_, err := schema.Ensure(db)
+	assert.NoError(t, err)
 
 	schema, _ = newSchemaAndDB(t)
-	err := schema.Ensure(db)
+	_, err = schema.Ensure(db)
 	assert.NotNil(t, err)
 	assert.EqualError(t, err, "schema version '1' is more recent than expected '0'")
 }
 
+// If a "fresh" SQL statement for creating the schema from scratch is provided,
+// but it fails to run, an error is returned.
+func TestSchemaEnsure_FreshStatementError(t *testing.T) {
+	schema, db := newSchemaAndDB(t)
+	schema.Add(updateNoop)
+	schema.Fresh("garbage")
+
+	_, err := schema.Ensure(db)
+	assert.NotNil(t, err)
+	assert.Contains(t, err.Error(), "cannot apply fresh schema")
+}
+
 // If the database schema contains "holes" in the applied versions, an error is
 // returned.
 func TestSchemaEnsure_MissingVersion(t *testing.T) {
 	schema, db := newSchemaAndDB(t)
 	schema.Add(updateNoop)
-	assert.NoError(t, schema.Ensure(db))
+	_, err := schema.Ensure(db)
+	assert.NoError(t, err)
 
-	_, err := db.Exec(`INSERT INTO schema (version, updated_at) VALUES (3, strftime("%s"))`)
+	_, err = db.Exec(`INSERT INTO schema (version, updated_at) VALUES (3, strftime("%s"))`)
+	assert.NoError(t, err)
 
 	schema.Add(updateNoop)
 	schema.Add(updateNoop)
 
-	err = schema.Ensure(db)
+	_, err = schema.Ensure(db)
 	assert.NotNil(t, err)
 	assert.EqualError(t, err, "missing updates: 1 -> 3")
 }
@@ -67,7 +85,7 @@ func TestSchemaEnsure_MissingVersion(t *testing.T) {
 func TestSchemaEnsure_ZeroUpdates(t *testing.T) {
 	schema, db := newSchemaAndDB(t)
 
-	err := schema.Ensure(db)
+	_, err := schema.Ensure(db)
 	assert.NoError(t, err)
 
 	tx, err := db.Begin()
@@ -85,8 +103,9 @@ func TestSchemaEnsure_ApplyAllUpdates(t *testing.T) {
 	schema.Add(updateCreateTable)
 	schema.Add(updateInsertValue)
 
-	err := schema.Ensure(db)
+	initial, err := schema.Ensure(db)
 	assert.NoError(t, err)
+	assert.Equal(t, 0, initial)
 
 	tx, err := db.Begin()
 	assert.NoError(t, err)
@@ -110,17 +129,20 @@ func TestSchemaEnsure_ApplyAfterInitialDumpCreation(t *testing.T) {
 	schema, db := newSchemaAndDB(t)
 	schema.Add(updateCreateTable)
 	schema.Add(updateAddColumn)
-	assert.NoError(t, schema.Ensure(db))
+	_, err := schema.Ensure(db)
+	assert.NoError(t, err)
 
 	dump, err := schema.Dump(db)
 	assert.NoError(t, err)
 
 	_, db = newSchemaAndDB(t)
-	_, err = db.Exec(dump)
+	schema.Fresh(dump)
+	_, err = schema.Ensure(db)
 	assert.NoError(t, err)
 
 	schema.Add(updateNoop)
-	assert.NoError(t, schema.Ensure(db))
+	_, err = schema.Ensure(db)
+	assert.NoError(t, err)
 
 	tx, err := db.Begin()
 	assert.NoError(t, err)
@@ -136,10 +158,13 @@ func TestSchemaEnsure_ApplyAfterInitialDumpCreation(t *testing.T) {
 func TestSchemaEnsure_OnlyApplyMissing(t *testing.T) {
 	schema, db := newSchemaAndDB(t)
 	schema.Add(updateCreateTable)
-	assert.NoError(t, schema.Ensure(db))
+	_, err := schema.Ensure(db)
+	assert.NoError(t, err)
 
 	schema.Add(updateInsertValue)
-	assert.NoError(t, schema.Ensure(db))
+	initial, err := schema.Ensure(db)
+	assert.NoError(t, err)
+	assert.Equal(t, 1, initial)
 
 	tx, err := db.Begin()
 	assert.NoError(t, err)
@@ -161,7 +186,7 @@ func TestSchemaEnsure_FailingUpdate(t *testing.T) {
 	schema, db := newSchemaAndDB(t)
 	schema.Add(updateCreateTable)
 	schema.Add(updateBoom)
-	err := schema.Ensure(db)
+	_, err := schema.Ensure(db)
 	assert.EqualError(t, err, "failed to apply update 1: boom")
 
 	tx, err := db.Begin()
@@ -180,7 +205,7 @@ func TestSchemaEnsure_FailingHook(t *testing.T) {
 	schema, db := newSchemaAndDB(t)
 	schema.Add(updateCreateTable)
 	schema.Hook(func(int, *sql.Tx) error { return fmt.Errorf("boom") })
-	err := schema.Ensure(db)
+	_, err := schema.Ensure(db)
 	assert.EqualError(t, err, "failed to execute hook (version 0): boom")
 
 	tx, err := db.Begin()
@@ -193,19 +218,47 @@ func TestSchemaEnsure_FailingHook(t *testing.T) {
 	assert.NotContains(t, tables, "test")
 }
 
+// If the schema check callback returns ErrGracefulAbort, the process is
+// aborted, although every change performed so far gets still committed.
+func TestSchemaEnsure_CheckGracefulAbort(t *testing.T) {
+	check := func(current int, tx *sql.Tx) error {
+		_, err := tx.Exec("CREATE TABLE test (n INTEGER)")
+		require.NoError(t, err)
+		return schema.ErrGracefulAbort
+	}
+
+	schema, db := newSchemaAndDB(t)
+	schema.Add(updateCreateTable)
+	schema.Check(check)
+
+	_, err := schema.Ensure(db)
+	require.EqualError(t, err, "schema check gracefully aborted")
+
+	tx, err := db.Begin()
+	assert.NoError(t, err)
+
+	// The table created by the check function still got committed.
+	// to insert the row was not.
+	ids, err := query.SelectIntegers(tx, "SELECT n FROM test")
+	assert.NoError(t, err)
+	assert.Equal(t, []int{}, ids)
+}
+
 // The SQL text returns by Dump() can be used to create the schema from
 // scratch, without applying each individual update.
 func TestSchemaDump(t *testing.T) {
 	schema, db := newSchemaAndDB(t)
 	schema.Add(updateCreateTable)
 	schema.Add(updateAddColumn)
-	assert.NoError(t, schema.Ensure(db))
+	_, err := schema.Ensure(db)
+	assert.NoError(t, err)
 
 	dump, err := schema.Dump(db)
 	assert.NoError(t, err)
 
 	_, db = newSchemaAndDB(t)
-	_, err = db.Exec(dump)
+	schema.Fresh(dump)
+	_, err = schema.Ensure(db)
 	assert.NoError(t, err)
 
 	tx, err := db.Begin()
@@ -226,13 +279,62 @@ func TestSchemaDump(t *testing.T) {
 func TestSchemaDump_MissingUpdatees(t *testing.T) {
 	schema, db := newSchemaAndDB(t)
 	schema.Add(updateCreateTable)
-	assert.NoError(t, schema.Ensure(db))
+	_, err := schema.Ensure(db)
+	assert.NoError(t, err)
 	schema.Add(updateAddColumn)
 
-	_, err := schema.Dump(db)
+	_, err = schema.Dump(db)
 	assert.EqualError(t, err, "update level is 1, expected 2")
 }
 
+// After trimming a schema, only the updates up to the trim point are applied.
+func TestSchema_Trim(t *testing.T) {
+	updates := map[int]schema.Update{
+		1: updateCreateTable,
+		2: updateInsertValue,
+		3: updateAddColumn,
+	}
+	schema := schema.NewFromMap(updates)
+	trimmed := schema.Trim(2)
+	assert.Len(t, trimmed, 1)
+
+	db := newDB(t)
+	_, err := schema.Ensure(db)
+	require.NoError(t, err)
+
+	tx, err := db.Begin()
+	require.NoError(t, err)
+
+	versions, err := query.SelectIntegers(tx, "SELECT version FROM schema")
+	require.NoError(t, err)
+	assert.Equal(t, []int{1, 2}, versions)
+}
+
+// Exercise a given update in a schema.
+func TestSchema_ExeciseUpdate(t *testing.T) {
+	updates := map[int]schema.Update{
+		1: updateCreateTable,
+		2: updateInsertValue,
+		3: updateAddColumn,
+	}
+
+	schema := schema.NewFromMap(updates)
+	db, err := schema.ExerciseUpdate(2, nil)
+	require.NoError(t, err)
+
+	tx, err := db.Begin()
+	require.NoError(t, err)
+
+	// Update 2 has been applied.
+	ids, err := query.SelectIntegers(tx, "SELECT id FROM test")
+	require.NoError(t, err)
+	assert.Equal(t, []int{1}, ids)
+
+	// Update 3 has not been applied.
+	_, err = query.SelectStrings(tx, "SELECT name FROM test")
+	require.EqualError(t, err, "no such column: name")
+}
+
 // Return a new in-memory SQLite database.
 func newDB(t *testing.T) *sql.DB {
 	db, err := sql.Open("sqlite3", ":memory:")
