@@ -25,7 +25,7 @@ import (
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/logger"
 
-	log "gopkg.in/inconshreveable/log15.v2"
+	log "github.com/lxc/lxd/shared/log15"
 )
 
 var deviceSchedRebalance = make(chan []string, 2)
@@ -194,15 +194,6 @@ func deviceLoadGpu() ([]gpuDevice, []nvidiaGpuDevices, error) {
 				continue
 			}
 
-			if isCard {
-				// If it is a card it's minor number will be its id.
-				tmpGpu.id = strconv.Itoa(minorInt)
-				tmp := cardIds{
-					id:  tmpGpu.id,
-					pci: tmpGpu.pci,
-				}
-				cards = append(cards, tmp)
-			}
 			// Find matching /dev/nvidia* entry for /dev/dri/card*
 			if tmpGpu.isNvidiaGpu() && isCard {
 				if !isNvidia {
@@ -212,6 +203,10 @@ func deviceLoadGpu() ([]gpuDevice, []nvidiaGpuDevices, error) {
 				nvidiaPath := fmt.Sprintf("/proc/driver/nvidia/gpus/%s/information", tmpGpu.pci)
 				buf, err := ioutil.ReadFile(nvidiaPath)
 				if err != nil {
+					if os.IsNotExist(err) {
+						continue
+					}
+
 					return nil, nil, err
 				}
 				strBuf := strings.TrimSpace(string(buf))
@@ -240,6 +235,10 @@ func deviceLoadGpu() ([]gpuDevice, []nvidiaGpuDevices, error) {
 				stat := syscall.Stat_t{}
 				err = syscall.Stat(nvidiaPath, &stat)
 				if err != nil {
+					if os.IsNotExist(err) {
+						continue
+					}
+
 					return nil, nil, err
 				}
 				tmpGpu.nvidia.path = nvidiaPath
@@ -247,6 +246,16 @@ func deviceLoadGpu() ([]gpuDevice, []nvidiaGpuDevices, error) {
 				tmpGpu.nvidia.minor = shared.Minor(stat.Rdev)
 				tmpGpu.nvidia.id = strconv.Itoa(tmpGpu.nvidia.minor)
 			}
+
+			if isCard {
+				// If it is a card it's minor number will be its id.
+				tmpGpu.id = strconv.Itoa(minorInt)
+				tmp := cardIds{
+					id:  tmpGpu.id,
+					pci: tmpGpu.pci,
+				}
+				cards = append(cards, tmp)
+			}
 			gpus = append(gpus, tmpGpu)
 		}
 	}
@@ -442,6 +451,9 @@ func deviceNetlinkListener() (chan []string, chan []string, chan usbDevice, erro
 				}
 
 				devname, ok := props["DEVNAME"]
+				if !ok {
+					continue
+				}
 
 				busnum, ok := props["BUSNUM"]
 				if !ok {
@@ -530,7 +542,7 @@ func deviceTaskBalance(s *state.State) {
 	}
 
 	// Don't bother running when CGroup support isn't there
-	if !cgCpusetController {
+	if !s.OS.CGroupCPUsetController {
 		return
 	}
 
@@ -592,7 +604,7 @@ func deviceTaskBalance(s *state.State) {
 	}
 
 	// Iterate through the containers
-	containers, err := db.ContainersList(s.DB, db.CTypeRegular)
+	containers, err := s.DB.ContainersList(db.CTypeRegular)
 	if err != nil {
 		logger.Error("problem loading containers list", log.Ctx{"err": err})
 		return
@@ -714,11 +726,11 @@ func deviceTaskBalance(s *state.State) {
 
 func deviceNetworkPriority(s *state.State, netif string) {
 	// Don't bother running when CGroup support isn't there
-	if !cgNetPrioController {
+	if !s.OS.CGroupNetPrioController {
 		return
 	}
 
-	containers, err := db.ContainersList(s.DB, db.CTypeRegular)
+	containers, err := s.DB.ContainersList(db.CTypeRegular)
 	if err != nil {
 		return
 	}
@@ -749,7 +761,7 @@ func deviceNetworkPriority(s *state.State, netif string) {
 }
 
 func deviceUSBEvent(s *state.State, usb usbDevice) {
-	containers, err := db.ContainersList(s.DB, db.CTypeRegular)
+	containers, err := s.DB.ContainersList(db.CTypeRegular)
 	if err != nil {
 		logger.Error("problem loading containers list", log.Ctx{"err": err})
 		return
@@ -783,13 +795,13 @@ func deviceUSBEvent(s *state.State, usb usbDevice) {
 			}
 
 			if usb.action == "add" {
-				err := c.insertUnixDeviceNum(m, usb.major, usb.minor, usb.path)
+				err := c.insertUnixDeviceNum(fmt.Sprintf("unix.%s", name), m, usb.major, usb.minor, usb.path)
 				if err != nil {
 					logger.Error("failed to create usb device", log.Ctx{"err": err, "usb": usb, "container": c.Name()})
 					return
 				}
 			} else if usb.action == "remove" {
-				err := c.removeUnixDeviceNum(m, usb.major, usb.minor, usb.path)
+				err := c.removeUnixDeviceNum(fmt.Sprintf("unix.%s", name), m, usb.major, usb.minor, usb.path)
 				if err != nil {
 					logger.Error("failed to remove usb device", log.Ctx{"err": err, "usb": usb, "container": c.Name()})
 					return
@@ -817,7 +829,7 @@ func deviceEventListener(s *state.State) {
 				continue
 			}
 
-			if !cgCpusetController {
+			if !s.OS.CGroupCPUsetController {
 				continue
 			}
 
@@ -829,7 +841,7 @@ func deviceEventListener(s *state.State) {
 				continue
 			}
 
-			if !cgNetPrioController {
+			if !s.OS.CGroupNetPrioController {
 				continue
 			}
 
@@ -844,7 +856,7 @@ func deviceEventListener(s *state.State) {
 				continue
 			}
 
-			if !cgCpusetController {
+			if !s.OS.CGroupCPUsetController {
 				continue
 			}
 
@@ -1347,3 +1359,237 @@ func deviceLoadUsb() ([]usbDevice, error) {
 
 	return result, nil
 }
+
+const SCIB string = "/sys/class/infiniband"
+const SCNET string = "/sys/class/net"
+
+type IBF struct {
+	// port the function belongs to
+	Port int64
+
+	// name of the {physical,virtual} function
+	Fun string
+
+	// whether this is a physical (true) or virtual (false) function
+	PF bool
+
+	// device of the function
+	Device string
+
+	// uverb device of the function
+	PerPortDevices []string
+	PerFunDevices  []string
+}
+
+func deviceLoadInfiniband() (map[string]IBF, error) {
+	// check if there are any infiniband devices
+	fscib, err := os.Open(SCIB)
+	if err != nil {
+		if os.IsNotExist(err) {
+			return nil, os.ErrNotExist
+		}
+		return nil, err
+	}
+	defer fscib.Close()
+
+	// eg.g. mlx_i for i = 0, 1, ..., n
+	IBDevNames, err := fscib.Readdirnames(-1)
+	if err != nil {
+		return nil, err
+	}
+
+	if len(IBDevNames) == 0 {
+		return nil, os.ErrNotExist
+	}
+
+	// retrieve all network device names
+	fscnet, err := os.Open(SCNET)
+	if err != nil {
+		if os.IsNotExist(err) {
+			return nil, os.ErrNotExist
+		}
+		return nil, err
+	}
+	defer fscnet.Close()
+
+	// retrieve all network devices
+	NetDevNames, err := fscnet.Readdirnames(-1)
+	if err != nil {
+		return nil, err
+	}
+
+	if len(NetDevNames) == 0 {
+		return nil, os.ErrNotExist
+	}
+
+	UseableDevices := make(map[string]IBF)
+	for _, IBDevName := range IBDevNames {
+		IBDevResourceFile := fmt.Sprintf("/sys/class/infiniband/%s/device/resource", IBDevName)
+		IBDevResourceBuf, err := ioutil.ReadFile(IBDevResourceFile)
+		if err != nil {
+			return nil, err
+		}
+
+		for _, NetDevName := range NetDevNames {
+			NetDevResourceFile := fmt.Sprintf("/sys/class/net/%s/device/resource", NetDevName)
+			NetDevResourceBuf, err := ioutil.ReadFile(NetDevResourceFile)
+			if err != nil {
+				if os.IsNotExist(err) {
+					continue
+				}
+				return nil, err
+			}
+
+			// If the device and the VF have the same address space
+			// they belong together.
+			if bytes.Compare(IBDevResourceBuf, NetDevResourceBuf) != 0 {
+				continue
+			}
+
+			// Now let's find the ports.
+			IBDevID := fmt.Sprintf("/sys/class/net/%s/dev_id", NetDevName)
+			IBDevPort := fmt.Sprintf("/sys/class/net/%s/dev_port", NetDevName)
+			DevIDBuf, err := ioutil.ReadFile(IBDevID)
+			if err != nil {
+				if os.IsNotExist(err) {
+					continue
+				}
+				return nil, err
+			}
+
+			DevIDString := strings.TrimSpace(string(DevIDBuf))
+			DevIDPort, err := strconv.ParseInt(DevIDString, 0, 64)
+			if err != nil {
+				return nil, err
+			}
+
+			DevPort := int64(0)
+			DevPortBuf, err := ioutil.ReadFile(IBDevPort)
+			if err != nil {
+				if !os.IsNotExist(err) {
+					return nil, err
+				}
+			} else {
+				DevPortString := strings.TrimSpace(string(DevPortBuf))
+				DevPort, err = strconv.ParseInt(DevPortString, 0, 64)
+				if err != nil {
+					return nil, err
+				}
+			}
+
+			Port := DevIDPort
+			if DevPort > DevIDPort {
+				Port = DevPort
+			}
+			Port++
+
+			NewIBF := IBF{
+				Port:   Port,
+				Fun:    IBDevName,
+				Device: NetDevName,
+			}
+
+			// identify the /dev/infiniband/uverb<idx> device
+			tmp := []string{}
+			IBUverb := fmt.Sprintf("/sys/class/net/%s/device/infiniband_verbs", NetDevName)
+			fuverb, err := os.Open(IBUverb)
+			if err != nil {
+				if !os.IsNotExist(err) {
+					return nil, err
+				}
+			} else {
+				defer fuverb.Close()
+
+				// optional: retrieve all network devices
+				tmp, err = fuverb.Readdirnames(-1)
+				if err != nil {
+					return nil, err
+				}
+
+				if len(tmp) == 0 {
+					return nil, os.ErrNotExist
+				}
+			}
+			for _, v := range tmp {
+				if strings.Index(v, "-") != -1 {
+					return nil, fmt.Errorf("Infiniband character device \"%s\" contains \"-\". Cannot guarantee unique encoding", v)
+				}
+				NewIBF.PerPortDevices = append(NewIBF.PerPortDevices, v)
+			}
+
+			// identify the /dev/infiniband/ucm<idx> device
+			tmp = []string{}
+			IBcm := fmt.Sprintf("/sys/class/net/%s/device/infiniband_ucm", NetDevName)
+			fcm, err := os.Open(IBcm)
+			if err != nil {
+				if !os.IsNotExist(err) {
+					return nil, err
+				}
+			} else {
+				defer fcm.Close()
+
+				// optional: retrieve all network devices
+				tmp, err = fcm.Readdirnames(-1)
+				if err != nil {
+					return nil, err
+				}
+
+				if len(tmp) == 0 {
+					return nil, os.ErrNotExist
+				}
+			}
+			for _, v := range tmp {
+				if strings.Index(v, "-") != -1 {
+					return nil, fmt.Errorf("Infiniband character device \"%s\" contains \"-\". Cannot guarantee unique encoding", v)
+				}
+				devPath := fmt.Sprintf("/dev/infiniband/%s", v)
+				NewIBF.PerPortDevices = append(NewIBF.PerPortDevices, devPath)
+			}
+
+			// identify the /dev/infiniband/{issm,umad}<idx> devices
+			IBmad := fmt.Sprintf("/sys/class/net/%s/device/infiniband_mad", NetDevName)
+			ents, err := ioutil.ReadDir(IBmad)
+			if err != nil {
+				if !os.IsNotExist(err) {
+					return nil, err
+				}
+			} else {
+				for _, ent := range ents {
+					IBmadPort := fmt.Sprintf("%s/%s/port", IBmad, ent.Name())
+					portBuf, err := ioutil.ReadFile(IBmadPort)
+					if err != nil {
+						if !os.IsNotExist(err) {
+							return nil, err
+						}
+						continue
+					}
+
+					portStr := strings.TrimSpace(string(portBuf))
+					PortMad, err := strconv.ParseInt(portStr, 0, 64)
+					if err != nil {
+						return nil, err
+					}
+
+					if PortMad != NewIBF.Port {
+						continue
+					}
+
+					if strings.Index(ent.Name(), "-") != -1 {
+						return nil, fmt.Errorf("Infiniband character device \"%s\" contains \"-\". Cannot guarantee unique encoding", ent.Name())
+					}
+
+					NewIBF.PerFunDevices = append(NewIBF.PerFunDevices, ent.Name())
+				}
+			}
+
+			// figure out whether this is a physical function
+			IBPF := fmt.Sprintf("/sys/class/net/%s/device/physfn", NetDevName)
+			NewIBF.PF = !shared.PathExists(IBPF)
+
+			UseableDevices[NetDevName] = NewIBF
+		}
+	}
+
+	// check whether the device is an infiniband device
+	return UseableDevices, nil
+}
