@@ -2,17 +2,18 @@ package main
 
 import (
 	"fmt"
+
 	"github.com/lxc/lxd/client"
 )
 
-func cmdImport(args []string) error {
-	if len(args) < 2 {
+func cmdImport(args *Args) error {
+	if len(args.Params) < 1 {
 		return fmt.Errorf("please specify a container to import")
 	}
-	name := args[1]
+	name := args.Params[0]
 	req := map[string]interface{}{
 		"name":  name,
-		"force": *argForce,
+		"force": args.Force,
 	}
 
 	c, err := lxd.ConnectLXDUnix("", nil)
