@@ -87,16 +87,21 @@ int mkdir_p(const char *dir, mode_t mode)
 	return 0;
 }
 
-int copy(int target, int source)
+int copy(int target, int source, bool append)
 {
 	ssize_t n;
 	char buf[1024];
 
-	if (ftruncate(target, 0) < 0) {
+	if (!append && ftruncate(target, 0) < 0) {
 		error("error: truncate");
 		return -1;
 	}
 
+	if (append && lseek(target, 0, SEEK_END) < 0) {
+		error("error: seek");
+		return -1;
+	}
+
 	while ((n = read(source, buf, 1024)) > 0) {
 		if (write(target, buf, n) != n) {
 			error("error: write");
@@ -175,15 +180,16 @@ void attach_userns(int pid) {
 	}
 }
 
-int manip_file_in_ns(char *rootfs, int pid, char *host, char *container, bool is_put, uid_t uid, gid_t gid, mode_t mode, uid_t defaultUid, gid_t defaultGid, mode_t defaultMode) {
+int manip_file_in_ns(char *rootfs, int pid, char *host, char *container, bool is_put, char *type, uid_t uid, gid_t gid, mode_t mode, uid_t defaultUid, gid_t defaultGid, mode_t defaultMode, bool append) {
 	int host_fd = -1, container_fd = -1;
 	int ret = -1;
 	int container_open_flags;
 	struct stat st;
 	int exists = 1;
-	bool is_dir_manip = !strcmp(host, "");
+	bool is_dir_manip = type != NULL && !strcmp(type, "directory");
+	bool is_symlink_manip = type != NULL && !strcmp(type, "symlink");
 
-	if (!is_dir_manip) {
+	if (!is_dir_manip && !is_symlink_manip) {
 		host_fd = open(host, O_RDWR);
 		if (host_fd < 0) {
 			error("error: open");
@@ -236,6 +242,32 @@ int manip_file_in_ns(char *rootfs, int pid, char *host, char *container, bool is
 		return 0;
 	}
 
+	if (is_put && is_symlink_manip) {
+		if (mode == -1) {
+			mode = defaultMode;
+		}
+
+		if (uid == -1) {
+			uid = defaultUid;
+		}
+
+		if (gid == -1) {
+			gid = defaultGid;
+		}
+
+		if (symlink(host, container) < 0 && errno != EEXIST) {
+			error("error: symlink");
+			return -1;
+		}
+
+		if (fchownat(0, container, uid, gid, AT_SYMLINK_NOFOLLOW) < 0) {
+			error("error: chown");
+			return -1;
+		}
+
+		return 0;
+	}
+
 	if (stat(container, &st) < 0)
 		exists = 0;
 
@@ -273,7 +305,7 @@ int manip_file_in_ns(char *rootfs, int pid, char *host, char *container, bool is
 			}
 		}
 
-		if (copy(container_fd, host_fd) < 0) {
+		if (copy(container_fd, host_fd, append) < 0) {
 			error("error: copy");
 			goto close_container;
 		}
@@ -328,11 +360,12 @@ int manip_file_in_ns(char *rootfs, int pid, char *host, char *container, bool is
 				fprintf(stderr, "\n");
 			}
 
-			// container_fd is dead now that we fopendir'd it
+			closedir(fdir);
+			// container_fd is dead now that we fdopendir'd it
 			goto close_host;
 		} else {
 			fprintf(stderr, "type: file\n");
-			ret = copy(host_fd, container_fd);
+			ret = copy(host_fd, container_fd, false);
 		}
 		fprintf(stderr, "type: %s", S_ISDIR(st.st_mode) ? "directory" : "file");
 	}
@@ -393,21 +426,27 @@ void ensure_file(char *dest) {
 }
 
 void create(char *src, char *dest) {
+	char *dirdup;
 	char *destdirname;
+
 	struct stat sb;
 	if (stat(src, &sb) < 0) {
 		fprintf(stderr, "source %s does not exist\n", src);
 		_exit(1);
 	}
 
-	destdirname = strdup(dest);
-	destdirname = dirname(destdirname);
+	dirdup = strdup(dest);
+	if (!dirdup)
+		_exit(1);
+
+	destdirname = dirname(dirdup);
 
 	if (mkdir_p(destdirname, 0755) < 0) {
 		fprintf(stderr, "failed to create path: %s\n", destdirname);
-		free(destdirname);
+		free(dirdup);
 		_exit(1);
 	}
+	free(dirdup);
 
 	switch (sb.st_mode & S_IFMT) {
 	case S_IFDIR:
@@ -417,8 +456,6 @@ void create(char *src, char *dest) {
 		ensure_file(dest);
 		return;
 	}
-
-	free(destdirname);
 }
 
 void forkmount(char *buf, char *cur, ssize_t size) {
@@ -492,8 +529,9 @@ void forkdofile(char *buf, char *cur, bool is_put, ssize_t size) {
 	uid_t defaultUid = 0;
 	gid_t defaultGid = 0;
 	mode_t defaultMode = 0;
-	char *command = cur, *rootfs = NULL, *source = NULL, *target = NULL;
+	char *command = cur, *rootfs = NULL, *source = NULL, *target = NULL, *writeMode = NULL, *type = NULL;
 	pid_t pid;
+	bool append = false;
 
 	ADVANCE_ARG_REQUIRED();
 	rootfs = cur;
@@ -508,6 +546,9 @@ void forkdofile(char *buf, char *cur, bool is_put, ssize_t size) {
 	target = cur;
 
 	if (is_put) {
+		ADVANCE_ARG_REQUIRED();
+		type = cur;
+
 		ADVANCE_ARG_REQUIRED();
 		uid = atoi(cur);
 
@@ -525,9 +566,14 @@ void forkdofile(char *buf, char *cur, bool is_put, ssize_t size) {
 
 		ADVANCE_ARG_REQUIRED();
 		defaultMode = atoi(cur);
+
+		ADVANCE_ARG_REQUIRED();
+		if (strcmp(cur, "append") == 0) {
+			append = true;
+		}
 	}
 
-	_exit(manip_file_in_ns(rootfs, pid, source, target, is_put, uid, gid, mode, defaultUid, defaultGid, defaultMode));
+	_exit(manip_file_in_ns(rootfs, pid, source, target, is_put, type, uid, gid, mode, defaultUid, defaultGid, defaultMode, append));
 }
 
 void forkcheckfile(char *buf, char *cur, bool is_put, ssize_t size) {
