@@ -2,7 +2,6 @@ package main
 
 import (
 	"bufio"
-	"database/sql"
 	"encoding/binary"
 	"encoding/hex"
 	"fmt"
@@ -12,6 +11,7 @@ import (
 	"math/rand"
 	"net"
 	"os"
+	"os/exec"
 	"path/filepath"
 	"regexp"
 	"strconv"
@@ -29,8 +29,8 @@ import (
 
 var networkStaticLock sync.Mutex
 
-func networkAutoAttach(dbObj *sql.DB, devName string) error {
-	_, dbInfo, err := db.NetworkGetInterface(dbObj, devName)
+func networkAutoAttach(db *db.Node, devName string) error {
+	_, dbInfo, err := db.NetworkGetInterface(devName)
 	if err != nil {
 		// No match found, move on
 		return nil
@@ -77,8 +77,8 @@ func networkDetachInterface(netName string, devName string) error {
 	return nil
 }
 
-func networkGetInterfaces(dbObj *sql.DB) ([]string, error) {
-	networks, err := db.Networks(dbObj)
+func networkGetInterfaces(db *db.Node) ([]string, error) {
+	networks, err := db.Networks()
 	if err != nil {
 		return nil, err
 	}
@@ -103,7 +103,7 @@ func networkIsInUse(c container, name string) bool {
 			continue
 		}
 
-		if !shared.StringInSlice(d["nictype"], []string{"bridged", "macvlan", "physical"}) {
+		if !shared.StringInSlice(d["nictype"], []string{"bridged", "macvlan", "physical", "sriov"}) {
 			continue
 		}
 
@@ -183,8 +183,7 @@ func networkGetIP(subnet *net.IPNet, host int64) net.IP {
 
 	// Generate an IPv6
 	if subnet.IP.To4() == nil {
-		newIp := make(net.IP, 16)
-		newIp = bigIP.Bytes()
+		newIp := bigIP.Bytes()
 		return newIp
 	}
 
@@ -300,7 +299,7 @@ func networkInRoutingTable(subnet *net.IPNet) bool {
 		fields := strings.Fields(string(line))
 
 		// Get the IP
-		ip := net.IP{}
+		var ip net.IP
 		if filename == "ipv6_route" {
 			ip, err = hex.DecodeString(fields[0])
 			if err != nil {
@@ -316,7 +315,7 @@ func networkInRoutingTable(subnet *net.IPNet) bool {
 		}
 
 		// Get the mask
-		mask := net.IPMask{}
+		var mask net.IPMask
 		if filename == "ipv6_route" {
 			size, err := strconv.ParseInt(fmt.Sprintf("0x%s", fields[1]), 0, 64)
 			if err != nil {
@@ -473,7 +472,7 @@ func networkValidName(value string) error {
 	}
 
 	// Validate the character set
-	match, _ := regexp.MatchString("^[-a-zA-Z0-9]*$", value)
+	match, _ := regexp.MatchString("^[-_a-zA-Z0-9.]*$", value)
 	if !match {
 		return fmt.Errorf("Interface name contains invalid characters")
 	}
@@ -729,11 +728,13 @@ func networkKillDnsmasq(name string, reload bool) error {
 }
 
 func networkGetDnsmasqVersion() (*version.DottedVersion, error) {
-	output, err := shared.TryRunCommand("dnsmasq", "--version")
+	// Discard stderr on purpose (occasional linker errors)
+	output, err := exec.Command("dnsmasq", "--version").Output()
 	if err != nil {
-		return nil, fmt.Errorf("Failed to check dnsmasq version")
+		return nil, fmt.Errorf("Failed to check dnsmasq version: %v", err)
 	}
-	lines := strings.Split(output, " ")
+
+	lines := strings.Split(string(output), " ")
 	return version.NewDottedVersion(lines[2])
 }
 
@@ -743,16 +744,16 @@ func networkUpdateStatic(s *state.State, networkName string) error {
 	defer networkStaticLock.Unlock()
 
 	// Get all the containers
-	containers, err := db.ContainersList(s.DB, db.CTypeRegular)
+	containers, err := s.DB.ContainersList(db.CTypeRegular)
 	if err != nil {
 		return err
 	}
 
 	// Get all the networks
-	networks := []string{}
+	var networks []string
 	if networkName == "" {
 		var err error
-		networks, err = db.Networks(s.DB)
+		networks, err = s.DB.Networks()
 		if err != nil {
 			return err
 		}
