@@ -11,7 +11,6 @@ import (
 	"github.com/gorilla/mux"
 	_ "github.com/mattn/go-sqlite3"
 
-	"github.com/lxc/lxd/lxd/db"
 	"github.com/lxc/lxd/lxd/state"
 	"github.com/lxc/lxd/lxd/util"
 	"github.com/lxc/lxd/shared"
@@ -19,12 +18,12 @@ import (
 	"github.com/lxc/lxd/shared/logger"
 	"github.com/lxc/lxd/shared/version"
 
-	log "gopkg.in/inconshreveable/log15.v2"
+	log "github.com/lxc/lxd/shared/log15"
 )
 
 /* This is used for both profiles post and profile put */
 func profilesGet(d *Daemon, r *http.Request) Response {
-	results, err := db.Profiles(d.db)
+	results, err := d.db.Profiles()
 	if err != nil {
 		return SmartError(err)
 	}
@@ -67,7 +66,7 @@ func profilesPost(d *Daemon, r *http.Request) Response {
 		return BadRequest(fmt.Errorf("No name provided"))
 	}
 
-	_, profile, _ := db.ProfileGet(d.db, req.Name)
+	_, profile, _ := d.db.ProfileGet(req.Name)
 	if profile != nil {
 		return BadRequest(fmt.Errorf("The profile already exists"))
 	}
@@ -91,7 +90,7 @@ func profilesPost(d *Daemon, r *http.Request) Response {
 	}
 
 	// Update DB entry
-	_, err = db.ProfileCreate(d.db, req.Name, req.Description, req.Config, req.Devices)
+	_, err = d.db.ProfileCreate(req.Name, req.Description, req.Config, req.Devices)
 	if err != nil {
 		return SmartError(
 			fmt.Errorf("Error inserting %s into database: %s", req.Name, err))
@@ -106,12 +105,12 @@ var profilesCmd = Command{
 	post: profilesPost}
 
 func doProfileGet(s *state.State, name string) (*api.Profile, error) {
-	_, profile, err := db.ProfileGet(s.DB, name)
+	_, profile, err := s.DB.ProfileGet(name)
 	if err != nil {
 		return nil, err
 	}
 
-	cts, err := db.ProfileContainersGet(s.DB, name)
+	cts, err := s.DB.ProfileContainersGet(name)
 	if err != nil {
 		return nil, err
 	}
@@ -140,7 +139,7 @@ func profileGet(d *Daemon, r *http.Request) Response {
 func getContainersWithProfile(s *state.State, profile string) []container {
 	results := []container{}
 
-	output, err := db.ProfileContainersGet(s.DB, profile)
+	output, err := s.DB.ProfileContainersGet(profile)
 	if err != nil {
 		return results
 	}
@@ -160,7 +159,7 @@ func getContainersWithProfile(s *state.State, profile string) []container {
 func profilePut(d *Daemon, r *http.Request) Response {
 	// Get the profile
 	name := mux.Vars(r)["name"]
-	id, profile, err := db.ProfileGet(d.db, name)
+	id, profile, err := d.db.ProfileGet(name)
 	if err != nil {
 		return SmartError(fmt.Errorf("Failed to retrieve profile='%s'", name))
 	}
@@ -183,7 +182,7 @@ func profilePut(d *Daemon, r *http.Request) Response {
 func profilePatch(d *Daemon, r *http.Request) Response {
 	// Get the profile
 	name := mux.Vars(r)["name"]
-	id, profile, err := db.ProfileGet(d.db, name)
+	id, profile, err := d.db.ProfileGet(name)
 	if err != nil {
 		return SmartError(fmt.Errorf("Failed to retrieve profile='%s'", name))
 	}
@@ -261,7 +260,7 @@ func profilePost(d *Daemon, r *http.Request) Response {
 	}
 
 	// Check that the name isn't already in use
-	id, _, _ := db.ProfileGet(d.db, req.Name)
+	id, _, _ := d.db.ProfileGet(req.Name)
 	if id > 0 {
 		return Conflict
 	}
@@ -274,7 +273,7 @@ func profilePost(d *Daemon, r *http.Request) Response {
 		return BadRequest(fmt.Errorf("Invalid profile name '%s'", req.Name))
 	}
 
-	err := db.ProfileUpdate(d.db, name, req.Name)
+	err := d.db.ProfileUpdate(name, req.Name)
 	if err != nil {
 		return SmartError(err)
 	}
@@ -296,7 +295,7 @@ func profileDelete(d *Daemon, r *http.Request) Response {
 		return BadRequest(fmt.Errorf("Profile is currently in use"))
 	}
 
-	err = db.ProfileDelete(d.db, name)
+	err = d.db.ProfileDelete(name)
 	if err != nil {
 		return SmartError(err)
 	}
