@@ -27,16 +27,33 @@ type Response interface {
 // Sync response
 type syncResponse struct {
 	success  bool
+	etag     interface{}
 	metadata interface{}
 	location string
+	headers  map[string]string
 }
 
 func (r *syncResponse) Render(w http.ResponseWriter) error {
+	// Set an appropriate ETag header
+	if r.etag != nil {
+		etag, err := util.EtagHash(r.etag)
+		if err == nil {
+			w.Header().Set("ETag", etag)
+		}
+	}
+
+	// Prepare the JSON response
 	status := api.Success
 	if !r.success {
 		status = api.Failure
 	}
 
+	if r.headers != nil {
+		for h, v := range r.headers {
+			w.Header().Set(h, v)
+		}
+	}
+
 	if r.location != "" {
 		w.Header().Set("Location", r.location)
 		w.WriteHeader(201)
@@ -65,10 +82,18 @@ func SyncResponse(success bool, metadata interface{}) Response {
 	return &syncResponse{success: success, metadata: metadata}
 }
 
+func SyncResponseETag(success bool, metadata interface{}, etag interface{}) Response {
+	return &syncResponse{success: success, metadata: metadata, etag: etag}
+}
+
 func SyncResponseLocation(success bool, metadata interface{}, location string) Response {
 	return &syncResponse{success: success, metadata: metadata, location: location}
 }
 
+func SyncResponseHeaders(success bool, metadata interface{}, headers map[string]string) Response {
+	return &syncResponse{success: success, metadata: metadata, headers: headers}
+}
+
 var EmptySyncResponse = &syncResponse{success: true, metadata: make(map[string]interface{})}
 
 // File transfer response
@@ -282,6 +307,10 @@ func InternalError(err error) Response {
 	return &errorResponse{http.StatusInternalServerError, err.Error()}
 }
 
+func PreconditionFailed(err error) Response {
+	return &errorResponse{http.StatusPreconditionFailed, err.Error()}
+}
+
 /*
  * SmartError returns the right error message based on err.
  */
