@@ -1,8 +1,8 @@
 package state
 
 import (
-	"database/sql"
-
+	"github.com/lxc/lxd/lxd/db"
+	"github.com/lxc/lxd/lxd/maas"
 	"github.com/lxc/lxd/lxd/sys"
 )
 
@@ -10,15 +10,17 @@ import (
 // and the operating system. It's typically used by model entities such as
 // containers, volumes, etc. in order to perform changes.
 type State struct {
-	DB *sql.DB
-	OS *sys.OS
+	DB   *db.Node
+	MAAS *maas.Controller
+	OS   *sys.OS
 }
 
 // NewState returns a new State object with the given database and operating
 // system components.
-func NewState(db *sql.DB, os *sys.OS) *State {
+func NewState(db *db.Node, maas *maas.Controller, os *sys.OS) *State {
 	return &State{
-		DB: db,
-		OS: os,
+		DB:   db,
+		MAAS: maas,
+		OS:   os,
 	}
 }
