@@ -63,17 +63,16 @@ func getCustomUmountLockID(poolName string, volumeName string) string {
 
 // Simply cache used to storage the activated drivers on this LXD instance. This
 // allows us to avoid querying the database everytime and API call is made.
-var storagePoolDriversCacheInitialized bool
 var storagePoolDriversCacheVal atomic.Value
 var storagePoolDriversCacheLock sync.Mutex
 
-func readStoragePoolDriversCache() []string {
+func readStoragePoolDriversCache() map[string]string {
 	drivers := storagePoolDriversCacheVal.Load()
 	if drivers == nil {
-		return []string{}
+		return map[string]string{}
 	}
 
-	return drivers.([]string)
+	return drivers.(map[string]string)
 }
 
 // storageType defines the type of a storage
@@ -144,6 +143,7 @@ type storage interface {
 	StoragePoolDelete() error
 	StoragePoolMount() (bool, error)
 	StoragePoolUmount() (bool, error)
+	StoragePoolResources() (*api.ResourcesStoragePool, error)
 	StoragePoolUpdate(writable *api.StoragePoolPut, changedConfig []string) error
 	GetStoragePoolWritable() api.StoragePoolPut
 	SetStoragePoolWritable(writable *api.StoragePoolPut)
@@ -154,6 +154,7 @@ type storage interface {
 	StoragePoolVolumeMount() (bool, error)
 	StoragePoolVolumeUmount() (bool, error)
 	StoragePoolVolumeUpdate(writable *api.StorageVolumePut, changedConfig []string) error
+	StoragePoolVolumeRename(newName string) error
 	GetStoragePoolVolumeWritable() api.StorageVolumePut
 	SetStoragePoolVolumeWritable(writable *api.StorageVolumePut)
 
@@ -171,7 +172,7 @@ type storage interface {
 	ContainerRename(container container, newName string) error
 	ContainerRestore(container container, sourceContainer container) error
 	ContainerGetUsage(container container) (int64, error)
-	GetContainerPoolInfo() (int64, string)
+	GetContainerPoolInfo() (int64, string, string)
 	ContainerStorageReady(name string) bool
 
 	ContainerSnapshotCreate(target container, source container) error
@@ -282,7 +283,7 @@ func storageCoreInit(driver string) (storage, error) {
 
 func storageInit(s *state.State, poolName string, volumeName string, volumeType int) (storage, error) {
 	// Load the storage pool.
-	poolID, pool, err := db.StoragePoolGet(s.DB, poolName)
+	poolID, pool, err := s.DB.StoragePoolGet(poolName)
 	if err != nil {
 		return nil, err
 	}
@@ -297,7 +298,7 @@ func storageInit(s *state.State, poolName string, volumeName string, volumeType
 	// Load the storage volume.
 	volume := &api.StorageVolume{}
 	if volumeName != "" && volumeType >= 0 {
-		_, volume, err = db.StoragePoolVolumeGetType(s.DB, volumeName, volumeType, poolID)
+		_, volume, err = s.DB.StoragePoolVolumeGetType(volumeName, volumeType, poolID)
 		if err != nil {
 			return nil, err
 		}
@@ -315,6 +316,7 @@ func storageInit(s *state.State, poolName string, volumeName string, volumeType
 		btrfs.pool = pool
 		btrfs.volume = volume
 		btrfs.s = s
+		btrfs.db = s.DB
 		err = btrfs.StoragePoolInit()
 		if err != nil {
 			return nil, err
@@ -326,6 +328,7 @@ func storageInit(s *state.State, poolName string, volumeName string, volumeType
 		dir.pool = pool
 		dir.volume = volume
 		dir.s = s
+		dir.db = s.DB
 		err = dir.StoragePoolInit()
 		if err != nil {
 			return nil, err
@@ -337,6 +340,7 @@ func storageInit(s *state.State, poolName string, volumeName string, volumeType
 		ceph.pool = pool
 		ceph.volume = volume
 		ceph.s = s
+		ceph.db = s.DB
 		err = ceph.StoragePoolInit()
 		if err != nil {
 			return nil, err
@@ -348,6 +352,7 @@ func storageInit(s *state.State, poolName string, volumeName string, volumeType
 		lvm.pool = pool
 		lvm.volume = volume
 		lvm.s = s
+		lvm.db = s.DB
 		err = lvm.StoragePoolInit()
 		if err != nil {
 			return nil, err
@@ -359,6 +364,7 @@ func storageInit(s *state.State, poolName string, volumeName string, volumeType
 		mock.pool = pool
 		mock.volume = volume
 		mock.s = s
+		mock.db = s.DB
 		err = mock.StoragePoolInit()
 		if err != nil {
 			return nil, err
@@ -370,6 +376,7 @@ func storageInit(s *state.State, poolName string, volumeName string, volumeType
 		zfs.pool = pool
 		zfs.volume = volume
 		zfs.s = s
+		zfs.db = s.DB
 		err = zfs.StoragePoolInit()
 		if err != nil {
 			return nil, err
@@ -510,11 +517,11 @@ func storagePoolVolumeAttachInit(s *state.State, poolName string, volumeName str
 
 	st.SetStoragePoolVolumeWritable(&poolVolumePut)
 
-	poolID, err := db.StoragePoolGetID(s.DB, poolName)
+	poolID, err := s.DB.StoragePoolGetID(poolName)
 	if err != nil {
 		return nil, err
 	}
-	err = db.StoragePoolVolumeUpdate(s.DB, volumeName, volumeType, poolID, poolVolumePut.Description, poolVolumePut.Config)
+	err = s.DB.StoragePoolVolumeUpdate(volumeName, volumeType, poolID, poolVolumePut.Description, poolVolumePut.Config)
 	if err != nil {
 		return nil, err
 	}
@@ -537,7 +544,7 @@ func storagePoolVolumeContainerCreateInit(s *state.State, poolName string, conta
 
 func storagePoolVolumeContainerLoadInit(s *state.State, containerName string) (storage, error) {
 	// Get the storage pool of a given container.
-	poolName, err := db.ContainerPool(s.DB, containerName)
+	poolName, err := s.DB.ContainerPool(containerName)
 	if err != nil {
 		return nil, err
 	}
@@ -803,7 +810,7 @@ func StorageProgressWriter(op *operation, key string, description string) func(i
 }
 
 func SetupStorageDriver(s *state.State, forceCheck bool) error {
-	pools, err := db.StoragePools(s.DB)
+	pools, err := s.DB.StoragePools()
 	if err != nil {
 		if err == db.NoSuchObjectError {
 			logger.Debugf("No existing storage pools detected.")
@@ -820,7 +827,7 @@ func SetupStorageDriver(s *state.State, forceCheck bool) error {
 	// but the upgrade somehow got messed up then there will be no
 	// "storage_api" entry in the db.
 	if len(pools) > 0 && !forceCheck {
-		appliedPatches, err := db.Patches(s.DB)
+		appliedPatches, err := s.DB.Patches()
 		if err != nil {
 			return err
 		}
@@ -846,6 +853,12 @@ func SetupStorageDriver(s *state.State, forceCheck bool) error {
 		}
 	}
 
+	// Update the storage drivers cache in api_1.0.go.
+	storagePoolDriversCacheUpdate(s.DB)
+	return nil
+}
+
+func storagePoolDriversCacheUpdate(dbNode *db.Node) {
 	// Get a list of all storage drivers currently in use
 	// on this LXD instance. Only do this when we do not already have done
 	// this once to avoid unnecessarily querying the db. All subsequent
@@ -855,18 +868,27 @@ func SetupStorageDriver(s *state.State, forceCheck bool) error {
 	// copy-on-write semantics without locking in the read case seems
 	// appropriate. (Should be cheaper then querying the db all the time,
 	// especially if we keep adding more storage drivers.)
-	if !storagePoolDriversCacheInitialized {
-		tmp, err := db.StoragePoolsGetDrivers(s.DB)
-		if err != nil && err != db.NoSuchObjectError {
-			return nil
-		}
 
-		storagePoolDriversCacheLock.Lock()
-		storagePoolDriversCacheVal.Store(tmp)
-		storagePoolDriversCacheLock.Unlock()
-
-		storagePoolDriversCacheInitialized = true
+	drivers, err := dbNode.StoragePoolsGetDrivers()
+	if err != nil && err != db.NoSuchObjectError {
+		return
 	}
 
-	return nil
+	data := map[string]string{}
+	for _, driver := range drivers {
+		// Initialize a core storage interface for the given driver.
+		sCore, err := storageCoreInit(driver)
+		if err != nil {
+			continue
+		}
+
+		// Grab the version
+		data[driver] = sCore.GetStorageTypeVersion()
+	}
+
+	storagePoolDriversCacheLock.Lock()
+	storagePoolDriversCacheVal.Store(data)
+	storagePoolDriversCacheLock.Unlock()
+
+	return
 }
