@@ -11,6 +11,7 @@ import (
 	"github.com/lxc/lxd/lxd/state"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/logger"
+	"github.com/lxc/lxd/shared/version"
 )
 
 func (s *storageLvm) lvExtend(lvPath string, lvSize int64, fsType string, fsMntPoint string, volumeType int, data interface{}) error {
@@ -56,20 +57,7 @@ func (s *storageLvm) lvExtend(lvPath string, lvSize int64, fsType string, fsMntP
 			`volume type %d`, volumeType)
 	}
 
-	switch fsType {
-	case "xfs":
-		msg, err = shared.TryRunCommand("xfs_growfs", fsMntPoint)
-	default:
-		// default = ext4
-		msg, err = shared.TryRunCommand("resize2fs", lvPath)
-	}
-	if err != nil {
-		logger.Errorf("could not extend underlying %s filesystem for LV \"%s\": %s", fsType, lvPath, msg)
-		return fmt.Errorf("could not extend underlying %s filesystem for LV \"%s\": %s", fsType, lvPath, msg)
-	}
-
-	logger.Debugf("extended underlying %s filesystem for LV \"%s\"", fsType, lvPath)
-	return nil
+	return growFileSystem(fsType, lvPath, fsMntPoint)
 }
 
 func (s *storageLvm) lvReduce(lvPath string, lvSize int64, fsType string, fsMntPoint string, volumeType int, data interface{}) error {
@@ -82,54 +70,15 @@ func (s *storageLvm) lvReduce(lvPath string, lvSize int64, fsType string, fsMntP
 		return fmt.Errorf(`The size of the storage volume would be ` +
 			`less than 1MB`)
 	}
-
-	lvSizeString := strconv.FormatInt(lvSize, 10)
-	switch fsType {
-	case "xfs":
-		logger.Errorf("xfs filesystems cannot be shrunk: dump, mkfs, and restore are required")
-		return fmt.Errorf("xfs filesystems cannot be shrunk: dump, mkfs, and restore are required")
-	default:
-		// default = ext4
-		switch volumeType {
-		case storagePoolVolumeTypeContainer:
-			c := data.(container)
-			ourMount, err := c.StorageStop()
-			if err != nil {
-				return err
-			}
-			if !ourMount {
-				defer c.StorageStart()
-			}
-		case storagePoolVolumeTypeCustom:
-			ourMount, err := s.StoragePoolVolumeUmount()
-			if err != nil {
-				return err
-			}
-			if !ourMount {
-				defer s.StoragePoolVolumeMount()
-			}
-		default:
-			return fmt.Errorf(`Resizing not implemented for `+
-				`storage volume type %d`, volumeType)
-		}
-
-		msg, err = shared.TryRunCommand("e2fsck", "-f", "-y", lvPath)
-		if err != nil {
-			return err
-		}
-
-		// don't assume resize2fs semantics are sane (because they
-		// aren't)
-		kbSize := lvSize / 1024
-		ext4LvSizeString := strconv.FormatInt(kbSize, 10)
-		ext4LvSizeString += "K"
-		msg, err = shared.TryRunCommand("resize2fs", lvPath, ext4LvSizeString)
-		if err != nil {
-			logger.Errorf("could not reduce underlying %s filesystem for LV \"%s\": %s", fsType, lvPath, msg)
-			return fmt.Errorf("could not reduce underlying %s filesystem for LV \"%s\": %s", fsType, lvPath, msg)
-		}
+	cleanupFunc, err := shrinkVolumeFilesystem(s, volumeType, fsType, lvPath, fsMntPoint, lvSize, data)
+	if cleanupFunc != nil {
+		defer cleanupFunc()
+	}
+	if err != nil {
+		return err
 	}
 
+	lvSizeString := strconv.FormatInt(lvSize, 10)
 	msg, err = shared.TryRunCommand(
 		"lvreduce",
 		"-L", lvSizeString+"B",
@@ -223,7 +172,7 @@ func (s *storageLvm) renameLVByPath(oldName string, newName string, volumeType s
 	return lvmLVRename(poolName, oldLvmName, newLvmName)
 }
 
-func (s *storageLvm) removeLV(vgName string, volumeType string, lvName string) error {
+func removeLV(vgName string, volumeType string, lvName string) error {
 	lvmVolumePath := getLvmDevPath(vgName, volumeType, lvName)
 	output, err := shared.TryRunCommand("lvremove", "-f", lvmVolumePath)
 
@@ -346,14 +295,23 @@ func (s *storageLvm) copyContainerThinpool(target container, source container, r
 	containerLvmName := containerNameToLVName(containerName)
 	containerLvDevPath := getLvmDevPath(poolName,
 		storagePoolVolumeAPIEndpointContainers, containerLvmName)
-	if LVFilesystem == "xfs" {
-		msg, err := xfsGenerateNewUUID(containerLvDevPath)
+	// If btrfstune sees two btrfs filesystems with the same UUID it
+	// gets confused and wants both of them unmounted. So unmount
+	// the source as well.
+	if LVFilesystem == "btrfs" {
+		ourUmount, err := s.ContainerUmount(source.Name(), source.Path())
 		if err != nil {
-			logger.Errorf(`Failed to create new xfs UUID for `+
-				`container "%s" on storage pool "%s": %s`,
-				containerName, s.pool.Name, msg)
 			return err
 		}
+		if ourUmount {
+			s.ContainerMount(source)
+		}
+	}
+
+	msg, err := fsGenerateNewUUID(LVFilesystem, containerLvDevPath)
+	if err != nil {
+		logger.Errorf("Failed to create new \"%s\" UUID for container \"%s\" on storage pool \"%s\": %s", LVFilesystem, containerName, s.pool.Name, msg)
+		return err
 	}
 
 	return nil
@@ -503,7 +461,7 @@ func (s *storageLvm) containerCreateFromImageLv(c container, fp string) error {
 
 	imagePath := shared.VarPath("images", fp)
 	containerMntPoint := getContainerMountPoint(s.pool.Name, containerName)
-	err = unpackImage(imagePath, containerMntPoint, storageTypeLvm)
+	err = unpackImage(imagePath, containerMntPoint, storageTypeLvm, s.s.OS.RunningInUserNS)
 	if err != nil {
 		logger.Errorf(`Failed to unpack image "%s" into non-thinpool `+
 			`LVM storage volume "%s" for container "%s" on `+
@@ -538,6 +496,21 @@ func (s *storageLvm) containerCreateFromImageThinLv(c container, fp string) erro
 
 		var imgerr error
 		ok, _ := storageLVExists(imageLvmDevPath)
+		if ok {
+			_, volume, err := s.s.DB.StoragePoolVolumeGetType(fp, db.StoragePoolVolumeTypeImage, s.poolID)
+			if err != nil {
+				return err
+			}
+			if volume.Config["block.filesystem"] != s.getLvmFilesystem() {
+				// The storage pool volume.blockfilesystem property has changed, re-import the image
+				err := s.ImageDelete(fp)
+				if err != nil {
+					return err
+				}
+				ok = false
+			}
+		}
+
 		if !ok {
 			imgerr = s.ImageCreate(fp)
 		}
@@ -711,7 +684,7 @@ func storageLVMThinpoolExists(vgName string, poolName string) (bool, error) {
 func storageLVMGetThinPoolUsers(s *state.State) ([]string, error) {
 	results := []string{}
 
-	cNames, err := db.ContainersList(s.DB, db.CTypeRegular)
+	cNames, err := s.DB.ContainersList(db.CTypeRegular)
 	if err != nil {
 		return results, err
 	}
@@ -729,7 +702,7 @@ func storageLVMGetThinPoolUsers(s *state.State) ([]string, error) {
 		}
 	}
 
-	imageNames, err := db.ImagesGet(s.DB, false)
+	imageNames, err := s.DB.ImagesGet(false)
 	if err != nil {
 		return results, err
 	}
@@ -803,6 +776,14 @@ func getLvmDevPath(lvmPool string, volumeType string, lvmVolume string) string {
 	return fmt.Sprintf("/dev/%s/%s_%s", lvmPool, volumeType, lvmVolume)
 }
 
+func getLVName(lvmPool string, volumeType string, lvmVolume string) string {
+	if volumeType == "" {
+		return fmt.Sprintf("%s/%s", lvmPool, lvmVolume)
+	}
+
+	return fmt.Sprintf("%s/%s_%s", lvmPool, volumeType, lvmVolume)
+}
+
 func getPrefixedLvName(volumeType string, lvmVolume string) string {
 	return fmt.Sprintf("%s_%s", volumeType, lvmVolume)
 }
@@ -811,10 +792,9 @@ func lvmCreateLv(vgName string, thinPoolName string, lvName string, lvFsType str
 	var output string
 	var err error
 
-	targetVg := vgName
 	lvmPoolVolumeName := getPrefixedLvName(volumeType, lvName)
 	if makeThinLv {
-		targetVg = fmt.Sprintf("%s/%s", vgName, thinPoolName)
+		targetVg := fmt.Sprintf("%s/%s", vgName, thinPoolName)
 		output, err = shared.TryRunCommand("lvcreate", "--thin", "-n", lvmPoolVolumeName, "--virtualsize", lvSize+"B", targetVg)
 	} else {
 		output, err = shared.TryRunCommand("lvcreate", "-n", lvmPoolVolumeName, "--size", lvSize+"B", vgName)
@@ -900,41 +880,20 @@ func createDefaultThinPool(sTypeVersion string, vgName string, thinPoolName stri
 	return nil
 }
 
-func versionSplit(versionString string) (int, int, int, error) {
-	fs := strings.Split(versionString, ".")
-	majs, mins, incs := fs[0], fs[1], fs[2]
-
-	maj, err := strconv.Atoi(majs)
-	if err != nil {
-		return 0, 0, 0, err
-	}
-	min, err := strconv.Atoi(mins)
-	if err != nil {
-		return 0, 0, 0, err
-	}
-	incs = strings.Split(incs, "(")[0]
-	inc, err := strconv.Atoi(incs)
-	if err != nil {
-		return 0, 0, 0, err
-	}
-
-	return maj, min, inc, nil
-}
-
 func lvmVersionIsAtLeast(sTypeVersion string, versionString string) (bool, error) {
-	lvmVersion := strings.Split(sTypeVersion, "/")[0]
+	lvmVersionString := strings.Split(sTypeVersion, "/")[0]
 
-	lvmMaj, lvmMin, lvmInc, err := versionSplit(lvmVersion)
+	lvmVersion, err := version.Parse(lvmVersionString)
 	if err != nil {
 		return false, err
 	}
 
-	inMaj, inMin, inInc, err := versionSplit(versionString)
+	inVersion, err := version.Parse(versionString)
 	if err != nil {
 		return false, err
 	}
 
-	if lvmMaj < inMaj || lvmMin < inMin || lvmInc < inInc {
+	if lvmVersion.Compare(inVersion) < 0 {
 		return false, nil
 	}
 
