@@ -1,7 +1,6 @@
 package main
 
 import (
-	"database/sql"
 	"fmt"
 	"strings"
 
@@ -63,7 +62,7 @@ func storagePoolUpdate(state *state.State, name, newDescription string, newConfi
 
 	// Update the database if something changed
 	if len(changedConfig) != 0 || newDescription != oldDescription {
-		err = db.StoragePoolUpdate(state.DB, name, newDescription, newConfig)
+		err = state.DB.StoragePoolUpdate(name, newDescription, newConfig)
 		if err != nil {
 			return err
 		}
@@ -81,15 +80,15 @@ func storagePoolUpdate(state *state.State, name, newDescription string, newConfi
 // /1.0/containers/alp1/snapshots/snap0
 // /1.0/images/cedce20b5b236f1071134beba7a5fd2aa923fda49eea4c66454dd559a5d6e906
 // /1.0/profiles/default
-func storagePoolUsedByGet(dbOb *sql.DB, poolID int64, poolName string) ([]string, error) {
+func storagePoolUsedByGet(dbObj *db.Node, poolID int64, poolName string) ([]string, error) {
 	// Retrieve all non-custom volumes that exist on this storage pool.
-	volumes, err := db.StoragePoolVolumesGet(dbOb, poolID, []int{storagePoolVolumeTypeContainer, storagePoolVolumeTypeImage, storagePoolVolumeTypeCustom})
+	volumes, err := dbObj.StoragePoolVolumesGet(poolID, []int{storagePoolVolumeTypeContainer, storagePoolVolumeTypeImage, storagePoolVolumeTypeCustom})
 	if err != nil && err != db.NoSuchObjectError {
 		return []string{}, err
 	}
 
 	// Retrieve all profiles that exist on this storage pool.
-	profiles, err := profilesUsingPoolGetNames(dbOb, poolName)
+	profiles, err := profilesUsingPoolGetNames(dbObj, poolName)
 
 	if err != nil {
 		return []string{}, err
@@ -129,16 +128,16 @@ func storagePoolUsedByGet(dbOb *sql.DB, poolID int64, poolName string) ([]string
 	return poolUsedBy, err
 }
 
-func profilesUsingPoolGetNames(dbOb *sql.DB, poolName string) ([]string, error) {
+func profilesUsingPoolGetNames(db *db.Node, poolName string) ([]string, error) {
 	usedBy := []string{}
 
-	profiles, err := db.Profiles(dbOb)
+	profiles, err := db.Profiles()
 	if err != nil {
 		return usedBy, err
 	}
 
 	for _, pName := range profiles {
-		_, profile, err := db.ProfileGet(dbOb, pName)
+		_, profile, err := db.ProfileGet(pName)
 		if err != nil {
 			return usedBy, err
 		}
@@ -165,7 +164,7 @@ func storagePoolDBCreate(s *state.State, poolName, poolDescription string, drive
 	}
 
 	// Check that the storage pool does not already exist.
-	_, err = db.StoragePoolGetID(s.DB, poolName)
+	_, err = s.DB.StoragePoolGetID(poolName)
 	if err == nil {
 		return fmt.Errorf("The storage pool already exists")
 	}
@@ -239,7 +238,7 @@ func storagePoolCreateInternal(state *state.State, poolName, poolDescription str
 	configDiff, _ := storageConfigDiff(config, postCreateConfig)
 	if len(configDiff) > 0 {
 		// Create the database entry for the storage pool.
-		err = db.StoragePoolUpdate(state.DB, poolName, poolDescription, postCreateConfig)
+		err = state.DB.StoragePoolUpdate(poolName, poolDescription, postCreateConfig)
 		if err != nil {
 			return fmt.Errorf("Error inserting %s into database: %s", poolName, err)
 		}
@@ -253,45 +252,28 @@ func storagePoolCreateInternal(state *state.State, poolName, poolDescription str
 
 // Helper around the low-level DB API, which also updates the driver names
 // cache.
-func dbStoragePoolCreateAndUpdateCache(dbObj *sql.DB, poolName string, poolDescription string, poolDriver string, poolConfig map[string]string) (int64, error) {
-	id, err := db.StoragePoolCreate(dbObj, poolName, poolDescription, poolDriver, poolConfig)
+func dbStoragePoolCreateAndUpdateCache(db *db.Node, poolName string, poolDescription string, poolDriver string, poolConfig map[string]string) (int64, error) {
+	id, err := db.StoragePoolCreate(poolName, poolDescription, poolDriver, poolConfig)
 	if err != nil {
 		return id, err
 	}
 
 	// Update the storage drivers cache in api_1.0.go.
-	storagePoolDriversCacheLock.Lock()
-	drivers := readStoragePoolDriversCache()
-	if !shared.StringInSlice(poolDriver, drivers) {
-		drivers = append(drivers, poolDriver)
-	}
-	storagePoolDriversCacheVal.Store(drivers)
-	storagePoolDriversCacheLock.Unlock()
+	storagePoolDriversCacheUpdate(db)
 
 	return id, nil
 }
 
 // Helper around the low-level DB API, which also updates the driver names
 // cache.
-func dbStoragePoolDeleteAndUpdateCache(dbObj *sql.DB, poolName string) error {
-	pool, err := db.StoragePoolDelete(dbObj, poolName)
+func dbStoragePoolDeleteAndUpdateCache(db *db.Node, poolName string) error {
+	_, err := db.StoragePoolDelete(poolName)
 	if err != nil {
 		return err
 	}
 
 	// Update the storage drivers cache in api_1.0.go.
-	storagePoolDriversCacheLock.Lock()
-	drivers := readStoragePoolDriversCache()
-	for i := 0; i < len(drivers); i++ {
-		if drivers[i] == pool.Driver {
-			drivers[i] = drivers[len(drivers)-1]
-			drivers[len(drivers)-1] = ""
-			drivers = drivers[:len(drivers)-1]
-			break
-		}
-	}
-	storagePoolDriversCacheVal.Store(drivers)
-	storagePoolDriversCacheLock.Unlock()
+	storagePoolDriversCacheUpdate(db)
 
 	return err
 }
