@@ -6,6 +6,7 @@ import (
 	"gopkg.in/yaml.v2"
 	"io"
 	"io/ioutil"
+	"os"
 	"strconv"
 	"strings"
 
@@ -20,6 +21,12 @@ type Context struct {
 	stderr io.Writer
 }
 
+// DefaultContext returns a new Context connected the stdin, stdout and stderr
+// streams.
+func DefaultContext() *Context {
+	return NewContext(os.Stdin, os.Stderr, os.Stdout)
+}
+
 // NewContext creates a new command context with the given parameters.
 func NewContext(stdin io.Reader, stdout, stderr io.Writer) *Context {
 	return &Context{
@@ -34,6 +41,11 @@ func (c *Context) Output(format string, a ...interface{}) {
 	fmt.Fprintf(c.stdout, format, a...)
 }
 
+// Error prints a message on standard error.
+func (c *Context) Error(format string, a ...interface{}) {
+	fmt.Fprintf(c.stderr, format, a...)
+}
+
 // AskBool asks a question an expect a yes/no answer.
 func (c *Context) AskBool(question string, defaultAnswer string) bool {
 	for {
