@@ -3,7 +3,7 @@
 package logging
 
 import (
-	log "gopkg.in/inconshreveable/log15.v2"
+	log "github.com/lxc/lxd/shared/log15"
 )
 
 // getSystemHandler on Windows does nothing.
