@@ -104,8 +104,13 @@ func IsUnixSocket(path string) bool {
 // On a normal system, this does nothing
 // When inside of a snap environment, returns the real path
 func HostPath(path string) string {
-	// Ignore relative paths
-	if len(path) == 0 || path[0] != os.PathSeparator {
+	// Ignore empty paths
+	if len(path) == 0 {
+		return path
+	}
+
+	// Don't prefix stdin/stdout
+	if path == "-" {
 		return path
 	}
 
@@ -115,8 +120,24 @@ func HostPath(path string) string {
 		return path
 	}
 
+	// Handle relative paths
+	if path[0] != os.PathSeparator {
+		// Use the cwd of the parent as snap-confine alters our own cwd on launch
+		ppid := os.Getppid()
+		if ppid < 1 {
+			return path
+		}
+
+		pwd, err := os.Readlink(fmt.Sprintf("/proc/%d/cwd", ppid))
+		if err != nil {
+			return path
+		}
+
+		path = filepath.Clean(strings.Join([]string{pwd, path}, string(os.PathSeparator)))
+	}
+
 	// Check if the path is already snap-aware
-	for _, prefix := range []string{"/snap", "/var/snap", "/var/lib/snapd"} {
+	for _, prefix := range []string{"/dev", "/snap", "/var/snap", "/var/lib/snapd"} {
 		if strings.HasPrefix(path, prefix) {
 			return path
 		}
@@ -857,3 +878,27 @@ func Round(x float64) int64 {
 
 	return int64(math.Floor(x + 0.5))
 }
+
+// WriteTempFile creates a temp file with the specified content
+func WriteTempFile(dir string, prefix string, content string) (string, error) {
+	f, err := ioutil.TempFile(dir, prefix)
+	if err != nil {
+		return "", err
+	}
+	defer f.Close()
+
+	_, err = f.WriteString(content)
+	return f.Name(), err
+}
+
+// EscapePathFstab escapes a path fstab-style.
+// This ensures that getmntent_r() and friends can correctly parse stuff like
+// /some/wacky path with spaces /some/wacky target with spaces
+func EscapePathFstab(path string) string {
+	r := strings.NewReplacer(
+		" ", "\\040",
+		"\t", "\\011",
+		"\n", "\\012",
+		"\\", "\\\\")
+	return r.Replace(path)
+}
