@@ -11,6 +11,9 @@ spawn_lxd() {
     lxddir=${1}
     shift
 
+    storage=${1}
+    shift
+
     # shellcheck disable=SC2153
     if [ "$LXD_BACKEND" = "random" ]; then
         lxd_backend="$(random_storage_backend)"
@@ -18,6 +21,11 @@ spawn_lxd() {
         lxd_backend="$LXD_BACKEND"
     fi
 
+    if [ "${LXD_BACKEND}" = "ceph" ] && [ -z "${LXD_CEPH_CLUSTER:-}" ]; then
+        echo "A cluster name must be specified when using the CEPH driver." >&2
+        exit 1
+    fi
+
     # Copy pre generated Certs
     cp deps/server.crt "${lxddir}"
     cp deps/server.key "${lxddir}"
@@ -54,8 +62,13 @@ spawn_lxd() {
         set -x
     fi
 
-    echo "==> Configuring storage backend"
-    "$lxd_backend"_configure "${lxddir}"
+    echo "==> Setting up networking"
+    LXD_DIR="${lxddir}" lxc profile device add default eth0 nic nictype=p2p name=eth0
+
+    if [ "${storage}" = true ]; then
+        echo "==> Configuring storage backend"
+        "$lxd_backend"_configure "${lxddir}"
+    fi
 }
 
 respawn_lxd() {
@@ -107,12 +120,23 @@ kill_lxd() {
             lxc image delete "${image}" --force-local || true
         done
 
+        # Delete all networks
+        echo "==> Deleting all networks"
+        for network in $(lxc network list --force-local | grep YES | grep "^| " | cut -d' ' -f2); do
+            lxc network delete "${network}" --force-local || true
+        done
+
         # Delete all profiles
         echo "==> Deleting all profiles"
-        for profile in $(lxc profile list --force-local); do
+        for profile in $(lxc profile list --force-local | tail -n+3 | grep "^| " | cut -d' ' -f2); do
             lxc profile delete "${profile}" --force-local || true
         done
 
+        echo "==> Deleting all storage pools"
+        for storage in $(lxc storage list --force-local | tail -n+3 | grep "^| " | cut -d' ' -f2); do
+            lxc storage delete "${storage}" --force-local || true
+        done
+
         echo "==> Checking for locked DB tables"
         for table in $(echo .tables | sqlite3 "${daemon_dir}/lxd.db"); do
             echo "SELECT * FROM ${table};" | sqlite3 "${daemon_dir}/lxd.db" >/dev/null
@@ -156,6 +180,8 @@ kill_lxd() {
         check_empty_table "${daemon_dir}/lxd.db" "containers_devices"
         check_empty_table "${daemon_dir}/lxd.db" "containers_devices_config"
         check_empty_table "${daemon_dir}/lxd.db" "containers_profiles"
+        check_empty_table "${daemon_dir}/lxd.db" "networks"
+        check_empty_table "${daemon_dir}/lxd.db" "networks_config"
         check_empty_table "${daemon_dir}/lxd.db" "images"
         check_empty_table "${daemon_dir}/lxd.db" "images_aliases"
         check_empty_table "${daemon_dir}/lxd.db" "images_properties"
@@ -164,6 +190,10 @@ kill_lxd() {
         check_empty_table "${daemon_dir}/lxd.db" "profiles_config"
         check_empty_table "${daemon_dir}/lxd.db" "profiles_devices"
         check_empty_table "${daemon_dir}/lxd.db" "profiles_devices_config"
+        check_empty_table "${daemon_dir}/lxd.db" "storage_pools"
+        check_empty_table "${daemon_dir}/lxd.db" "storage_pools_config"
+        check_empty_table "${daemon_dir}/lxd.db" "storage_volumes"
+        check_empty_table "${daemon_dir}/lxd.db" "storage_volumes_config"
     fi
 
     # teardown storage
@@ -236,6 +266,15 @@ cleanup_lxds() {
         kill_lxd "${daemon_dir}"
     done < "${test_dir}/daemons"
 
+    # Cleanup leftover networks
+    # shellcheck disable=SC2009
+    ps aux | grep "interface=lxdt$$ " | grep -v grep | awk '{print $2}' | while read -r line; do
+        kill -9 "${line}"
+    done
+    if [ -e "/sys/class/net/lxdt$$" ]; then
+        ip link del lxdt$$
+    fi
+
     # Wipe the test environment
     wipe "$test_dir"
 
