@@ -84,9 +84,9 @@ cleanup() {
 
   echo "==> Cleaning up"
 
+  kill_external_auth_daemon "$TEST_DIR"
   cleanup_lxds "$TEST_DIR"
 
-
   echo ""
   echo ""
   if [ "${TEST_RESULT}" != "success" ]; then
@@ -123,6 +123,8 @@ spawn_lxd "${LXD_DIR}" true
 LXD_ADDR=$(cat "${LXD_DIR}/lxd.addr")
 export LXD_ADDR
 
+start_external_auth_daemon "${LXD_DIR}"
+
 run_test() {
   TEST_CURRENT=${1}
   TEST_CURRENT_DESCRIPTION=${2:-${1}}
@@ -184,6 +186,10 @@ run_test test_storage_profiles "storage profiles"
 run_test test_container_import "container import"
 run_test test_storage_volume_attach "attaching storage volumes"
 run_test test_storage_driver_ceph "ceph storage driver"
+run_test test_resources "resources"
+run_test test_kernel_limits "kernel limits"
+run_test test_macaroon_auth "macaroon authentication"
+run_test test_console "console"
 
 # shellcheck disable=SC2034
 TEST_RESULT=success
