@@ -27,6 +27,11 @@ test_basic_usage() {
   lxc image alias delete foo
   lxc image alias delete bar
 
+  lxc image alias create foo "${sum}"
+  lxc image alias rename foo bar
+  lxc image alias list | grep -qv foo  # the old name is gone
+  lxc image alias delete bar
+
   # Test image list output formats (table & json)
   lxc image list --format table | grep -q testimage
   lxc image list --format json \
@@ -55,8 +60,8 @@ test_basic_usage() {
 
   # Test custom filename for image export
   lxc image export testimage "${LXD_DIR}/foo"
-  [ "${sum}" = "$(sha256sum "${LXD_DIR}/foo" | cut -d' ' -f1)" ]
-  rm "${LXD_DIR}/foo"
+  [ "${sum}" = "$(sha256sum "${LXD_DIR}/foo.tar.xz" | cut -d' ' -f1)" ]
+  rm "${LXD_DIR}/foo.tar.xz"
 
 
   # Test image export with a split image.
@@ -102,6 +107,11 @@ test_basic_usage() {
   lxc list | grep -v foo
   lxc list | grep bar
 
+  lxc rename bar foo
+  lxc list | grep -v bar
+  lxc list | grep foo
+  lxc rename foo bar
+
   # Test container copy
   lxc copy bar foo
   lxc delete foo
@@ -370,11 +380,11 @@ test_basic_usage() {
       aa_namespace="lxd-lxd-apparmor-test_<$(echo "${LXD_DIR}" | sed -e 's/\//-/g' -e 's/^.//')>"
       aa-status | grep -q ":${aa_namespace}:unconfined" || aa-status | grep -q ":${aa_namespace}://unconfined"
       lxc stop lxd-apparmor-test --force
-      ! aa-status | grep -q ":${aa_namespace}:"
+      ! aa-status | grep -q ":${aa_namespace}:" || false
     else
       aa-status | grep "lxd-lxd-apparmor-test_<${LXD_DIR}>"
       lxc stop lxd-apparmor-test --force
-      ! aa-status | grep -q "lxd-lxd-apparmor-test_<${LXD_DIR}>"
+      ! aa-status | grep -q "lxd-lxd-apparmor-test_<${LXD_DIR}>" || false
     fi
     lxc delete lxd-apparmor-test
     [ ! -f "${LXD_DIR}/security/apparmor/profiles/lxd-lxd-apparmor-test" ]
@@ -420,12 +430,25 @@ test_basic_usage() {
   lxc publish --force c3 --alias=image3
   # Delete multiple images with lxc delete and confirm they're deleted
   lxc image delete local:image1 local:image2 local:image3
-  ! lxc image list | grep -q image1
-  ! lxc image list | grep -q image2
-  ! lxc image list | grep -q image3
+  ! lxc image list | grep -q image1 || false
+  ! lxc image list | grep -q image2 || false
+  ! lxc image list | grep -q image3 || false
   # Cleanup the containers
   lxc delete --force c1 c2 c3
 
+  # Test --all flag
+  lxc init testimage c1
+  lxc init testimage c2
+  lxc start --all
+  lxc list | grep c1 | grep RUNNING
+  lxc list | grep c2 | grep RUNNING
+  ! lxc stop --all c1 || false
+  lxc stop --all
+  lxc list | grep c1 | grep STOPPED
+  lxc list | grep c2 | grep STOPPED
+  # Cleanup the containers
+  lxc delete --force c1 c2
+
   # Ephemeral
   lxc launch testimage foo -e
 
@@ -453,5 +476,5 @@ test_basic_usage() {
   sleep 2
 
   lxc stop foo --force || true
-  ! lxc list | grep -q foo
+  ! lxc list | grep -q foo || false
 }
