@@ -25,6 +25,11 @@ dounixdevtest() {
 }
 
 testunixdevs() {
+  if [ ! -e /dev/ttyS0 ] || [ ! -e /dev/ttyS1 ]; then
+     echo "==> SKIP: /dev/ttyS0 or /dev/ttyS1 are missing"
+     return
+  fi
+
   echo "Testing passing char device /dev/ttyS0"
   dounixdevtest path=/dev/ttyS0
 
@@ -116,7 +121,7 @@ test_config_profiles() {
 
   # setting an invalid config item should error out when setting it, not get
   # into the database and never let the user edit the container again.
-  ! lxc config set foo raw.lxc "lxc.notaconfigkey = invalid"
+  ! lxc config set foo raw.lxc lxc.notaconfigkey=invalid
 
   # check that various profile application mechanisms work
   lxc profile create one
@@ -150,6 +155,11 @@ test_config_profiles() {
   lxc profile create unconfined
   lxc profile set unconfined raw.lxc "lxc.aa_profile=unconfined"
   lxc profile assign foo onenic,unconfined
+  # test profile rename
+  lxc profile create foo
+  lxc profile rename foo bar
+  lxc profile list | grep -qv foo  # the old name is gone
+  lxc profile delete bar
 
   lxc config device list foo | grep mnt1
   lxc config device show foo | grep "/mnt1"
@@ -161,9 +171,9 @@ test_config_profiles() {
   # test live-adding a nic
   lxc start foo
   lxc exec foo -- cat /proc/self/mountinfo | grep -q "/mnt1.*ro,"
-  ! lxc config show foo | grep -q "raw.lxc"
+  ! lxc config show foo | grep -q "raw.lxc" || false
   lxc config show foo --expanded | grep -q "raw.lxc"
-  ! lxc config show foo | grep -v "volatile.eth0" | grep -q "eth0"
+  ! lxc config show foo | grep -v "volatile.eth0" | grep -q "eth0" || false
   lxc config show foo --expanded | grep -v "volatile.eth0" | grep -q "eth0"
   lxc config device add foo eth2 nic nictype=p2p name=eth10
   lxc exec foo -- /sbin/ifconfig -a | grep eth0
@@ -205,7 +215,7 @@ test_config_profiles() {
   bad=0
   lxc config set foo user.prop 2>/dev/null && bad=1
   if [ "${bad}" -eq 1 ]; then
-    echo "property set succeded when it shouldn't have"
+    echo "property set succeeded when it shouldn't have"
     false
   fi
 
@@ -288,7 +298,7 @@ test_container_metadata() {
     
     # templates can be removed
     lxc config template delete c my.tpl
-    ! (lxc config template list c | grep -q my.tpl)
+    ! lxc config template list c | grep -q my.tpl || false
 
     lxc delete c
 }
