@@ -15,6 +15,9 @@ test_remote_url() {
     urls="images.linuxcontainers.org https://images.linuxcontainers.org ${urls}"
   fi
 
+  # an invalid protocol returns an error
+  ! lxc_remote remote add test "${url}" --accept-certificate --password foo --protocol foo
+
   for url in ${urls}; do
     lxc_remote remote add test "${url}"
     lxc_remote remote remove test
@@ -84,8 +87,8 @@ test_remote_usage() {
 
   lxc_remote image export localhost:testimage "${LXD_DIR}/foo"
   lxc_remote image delete localhost:testimage
-  sum=$(sha256sum "${LXD_DIR}/foo" | cut -d' ' -f1)
-  lxc_remote image import "${LXD_DIR}/foo" localhost: --public
+  sum=$(sha256sum "${LXD_DIR}/foo.tar.xz" | cut -d' ' -f1)
+  lxc_remote image import "${LXD_DIR}/foo.tar.xz" localhost: --public
   lxc_remote image alias create localhost:testimage "${sum}"
 
   lxc_remote image delete "lxd2:${sum}" || true
