@@ -19,17 +19,49 @@ test_storage() {
   lxc storage show "$storage_pool" | grep -q 'description: foo'
 
   lxc storage volume create "$storage_pool" "$storage_volume"
-  if [ "$lxd_backend" != "dir" ] && [ "$lxd_backend" != "ceph" ]; then
-    # Test resizing/applying quota to a storage volume.
-    lxc storage volume set "$storage_pool" "$storage_volume" size 200MB
-    lxc storage volume unset "$storage_pool" "$storage_volume" size
-  fi
+  # Test resizing/applying quota to a storage volume's of type container fails.
+  ! lxc storage volume set "$storage_pool" "$storage_volume" size 200MB
+
   # Test setting description on a storage volume
   lxc storage volume show "$storage_pool" "$storage_volume" | sed 's/^description:.*/description: bar/' | lxc storage volume edit "$storage_pool" "$storage_volume"
   lxc storage volume show "$storage_pool" "$storage_volume" | grep -q 'description: bar'
   lxc storage volume delete "$storage_pool" "$storage_volume"
 
   lxc storage delete "$storage_pool"
+
+  # Test btrfs resize
+  if [ "$lxd_backend" = "lvm" ] || [ "$lxd_backend" = "ceph" ]; then
+      # shellcheck disable=2039
+      local btrfs_storage_pool btrfs_storage_volume
+      btrfs_storage_pool="lxdtest-$(basename "${LXD_DIR}")-pool-btrfs"
+      btrfs_storage_volume="${storage_pool}-vol"
+      lxc storage create "$btrfs_storage_pool" "$lxd_backend" volume.block.filesystem=btrfs volume.size=200MB
+      lxc storage volume create "$btrfs_storage_pool" "$btrfs_storage_volume"
+      lxc storage volume show "$btrfs_storage_pool" "$btrfs_storage_volume"
+      lxc storage volume set "$btrfs_storage_pool" "$btrfs_storage_volume" size 256MB
+      lxc storage volume delete "$btrfs_storage_pool" "$btrfs_storage_volume"
+
+      # Test generation of unique UUID.
+      lxc init testimage uuid1 -s "lxdtest-$(basename "${LXD_DIR}")-pool-btrfs"
+      POOL="lxdtest-$(basename "${LXD_DIR}")-pool-btrfs"
+      lxc copy uuid1 uuid2
+      lxc start uuid1
+      lxc start uuid2
+      lxc stop --force uuid1
+      lxc stop --force uuid2
+      if [ "$lxd_backend" = "lvm" ]; then
+        [ "$(blkid -s UUID -o value -p /dev/"${POOL}"/containers_uuid1)" != "$(blkid -s UUID -o value -p /dev/"${POOL}"/containers_uuid2)" ]
+      elif [ "$lxd_backend" = "ceph" ]; then
+        [ "$(blkid -s UUID -o value -p /dev/rbd/"${POOL}"/container_uuid1)" != "$(blkid -s UUID -o value -p /dev/rbd/"${POOL}"/container_uuid2)" ]
+      fi
+      lxc delete --force uuid1
+      lxc delete --force uuid2
+      lxc image delete testimage
+
+      lxc storage delete "$btrfs_storage_pool"
+  fi
+  ensure_import_testimage
+
   (
     set -e
     # shellcheck disable=2030
@@ -51,7 +83,7 @@ test_storage() {
       # Let LXD use an already existing storage pool.
       configure_loop_device loop_file_4 loop_device_4
       # shellcheck disable=SC2154
-      zpool create "lxdtest-$(basename "${LXD_DIR}")-pool9-existing-pool" "${loop_device_4}" -f -m none -O compression=on
+      zpool create -f -m none -O compression=on "lxdtest-$(basename "${LXD_DIR}")-pool9-existing-pool" "${loop_device_4}"
       lxc storage create "lxdtest-$(basename "${LXD_DIR}")-pool9" zfs source="lxdtest-$(basename "${LXD_DIR}")-pool9-existing-pool"
 
       # Let LXD create a new dataset and use as pool.
@@ -257,6 +289,8 @@ test_storage() {
       lxc storage volume attach "lxdtest-$(basename "${LXD_DIR}")-pool2" custom/c4pool2 c4pool2 testDevice /opt
       ! lxc storage volume attach "lxdtest-$(basename "${LXD_DIR}")-pool2" custom/c4pool2 c4pool2 testDevice2 /opt
       lxc storage volume detach "lxdtest-$(basename "${LXD_DIR}")-pool2" c4pool2 c4pool2
+      lxc storage volume rename "lxdtest-$(basename "${LXD_DIR}")-pool2" c4pool2 c4pool2-renamed
+      lxc storage volume rename "lxdtest-$(basename "${LXD_DIR}")-pool2" c4pool2-renamed c4pool2
     fi
 
     if [ "$lxd_backend" = "btrfs" ]; then
@@ -301,6 +335,8 @@ test_storage() {
       lxc storage volume attach "lxdtest-$(basename "${LXD_DIR}")-pool4" custom/c8pool4 c8pool4 testDevice /opt
       ! lxc storage volume attach "lxdtest-$(basename "${LXD_DIR}")-pool4" custom/c8pool4 c8pool4 testDevice2 /opt
       lxc storage volume detach "lxdtest-$(basename "${LXD_DIR}")-pool4" c8pool4 c8pool4 testDevice
+      lxc storage volume rename "lxdtest-$(basename "${LXD_DIR}")-pool4" c8pool4 c8pool4-renamed
+      lxc storage volume rename "lxdtest-$(basename "${LXD_DIR}")-pool4" c8pool4-renamed c8pool4
     fi
 
     lxc init testimage c9pool5 -s "lxdtest-$(basename "${LXD_DIR}")-pool5"
@@ -324,6 +360,8 @@ test_storage() {
     lxc storage volume attach "lxdtest-$(basename "${LXD_DIR}")-pool5" custom/c11pool5 c11pool5 testDevice /opt
     ! lxc storage volume attach "lxdtest-$(basename "${LXD_DIR}")-pool5" custom/c11pool5 c11pool5 testDevice2 /opt
     lxc storage volume detach "lxdtest-$(basename "${LXD_DIR}")-pool5" c11pool5 c11pool5 testDevice
+    lxc storage volume rename "lxdtest-$(basename "${LXD_DIR}")-pool5" c11pool5 c11pool5-renamed
+    lxc storage volume rename "lxdtest-$(basename "${LXD_DIR}")-pool5" c11pool5-renamed c11pool5
 
     if [ "$lxd_backend" = "lvm" ]; then
       lxc init testimage c10pool6 -s "lxdtest-$(basename "${LXD_DIR}")-pool6"
@@ -373,6 +411,13 @@ test_storage() {
       lxc launch testimage c12pool15 -s "lxdtest-$(basename "${LXD_DIR}")-non-thinpool-pool15"
       lxc list -c b c12pool15 | grep "lxdtest-$(basename "${LXD_DIR}")-non-thinpool-pool15"
 
+      # Test that changing block filesystem works
+      lxc storage set "lxdtest-$(basename "${LXD_DIR}")-pool6" volume.block.filesystem xfs
+      lxc init testimage c1pool6 -s "lxdtest-$(basename "${LXD_DIR}")-pool6"
+      lxc storage set "lxdtest-$(basename "${LXD_DIR}")-pool6" volume.block.filesystem btrfs
+      lxc storage set "lxdtest-$(basename "${LXD_DIR}")-pool6" volume.size 100MB
+      lxc init testimage c2pool6 -s "lxdtest-$(basename "${LXD_DIR}")-pool6"
+
       lxc storage volume create "lxdtest-$(basename "${LXD_DIR}")-pool6" c10pool6
       lxc storage volume attach "lxdtest-$(basename "${LXD_DIR}")-pool6" c10pool6 c10pool6 testDevice /opt
       ! lxc storage volume attach "lxdtest-$(basename "${LXD_DIR}")-pool6" c10pool6 c10pool6 testDevice2 /opt
@@ -527,6 +572,8 @@ test_storage() {
       lxc storage volume attach "lxdtest-$(basename "${LXD_DIR}")-pool9" custom/c18pool9 c18pool9 testDevice /opt
       ! lxc storage volume attach "lxdtest-$(basename "${LXD_DIR}")-pool9" custom/c18pool9 c18pool9 testDevice2 /opt
       lxc storage volume detach "lxdtest-$(basename "${LXD_DIR}")-pool9" c18pool9 c18pool9 testDevice
+      lxc storage volume rename "lxdtest-$(basename "${LXD_DIR}")-pool9" c18pool9 c18pool9-renamed
+      lxc storage volume rename "lxdtest-$(basename "${LXD_DIR}")-pool9" c18pool9-renamed c18pool9
     fi
 
     if [ "$lxd_backend" = "zfs" ]; then
@@ -562,6 +609,8 @@ test_storage() {
     lxc storage volume delete "lxdtest-$(basename "${LXD_DIR}")-pool5" c11pool5
 
     if [ "$lxd_backend" = "lvm" ]; then
+      lxc delete -f c1pool6
+      lxc delete -f c2pool6
       lxc delete -f c10pool6
       lxc delete -f c12pool6
 
